<?php

/*********************************************************************************
           TPLN  by HAYOUN Laurent

           website: http://tpln.sourceforge.net
           email: php_work@hotmail.com
           License : LGPL
**********************************************************************************/


// Inclusion du fichier de configuration et de Db
define('TPLN_PATH', dirname(__FILE__));
include('TPLN_Cfg.php');
include('lang/error_'.TPLN_LANG.'.inc.php'); // fichier de lang

// adoDB
include('adodb_lite/adodb.inc.php');

// plugins
include('plugin/dbtable/dbtable.class.php');
include('plugin/mail/mail.class.php');
include('plugin/form/form.class.php');


/******************* Structure d'un fichier ***************************************

   f - f_no - name                 - string     // nom de fichier
            - cache_name           - string     // nom du fichier en cache
            - buffer               - string     // contenu du fichier
            - items                - array      // contient tous les items
			- constant_items		   - array		// contient les constantes
            - php_items            - array      // contient les items avec $
            - cmd_items            - array      // contient tous les items includes
            - create_cached_file   - bool       // cre le fichier en cache ?
            - time_started         - long int   // dbut du chrono
            - cache_expire         - bool       // expiration du cache ?
            - execution_time       - long int   // temps d'execution
            - chrono_started       - long int   // chrono started

            Attention par reference pour les vrifications et performances

            - shortcut_blocs       - array      //
               |_ all              - array      // contient le nom de tous les blocs
               |_ used             - array      // contient les blocs appels par l'utilsateur
               |_ name             - none       // contient le nom du bloc
                    |_ items       - array      // contient les items

            - def_blocs            - array      // contiens tous les blocs structurs
                                                 // definis par l'utisateur lors d'un appel'
                                                 // voir plus bas leurs structures
*/
/******************* Structure d'un bloc ***************************************

  def_blocs - name                  - string     // nom du bloc
              |_ structure          - string     // contenu du bloc
              |_ parsed             - array      // contiens les sessions de blocs
              |_ is_looped          - boolean    // loop ?
              |_ children           - array      // contiens les bloc enfants

*************************************************************************************/
class TPLN extends Form
{
	var $def_tpl = array(); // liste des template dfini avec un nom
	var $f_no = -1; // indice de fichier
	var $vf_no = -1; // indice de fichier virtuel
	var $f = array(); // c'est un array qui contient toutes les proprits
	var $chrono_type = array(); // on veut le chrono de qui ALL ou pas ?
	var $HTMLCompress = 0; // active la compression HTML de sortie ?
	var $blocs_double_exceptions = array('start', 'end', 'pager', 'in', 'out', 'previous', 'next'); // nos blocs permis en double

	// DataBase
	var $db_index = -1;
	var $req_index = -1; // indice le jeux de resultats en cours
	var $db = array(); // objet contenant les informations de connection
	var $req = array(); // stockage des resultats
	var $DBLastError = ''; // dernire erreur SQL
	var $query_count = 0; // nombre de requetes executees

	// Xtra
    var $cons_query = array(); // la requete disloke est stocke ici
	var $NavColorFirst = ''; // couleur pour la navigation
	var $NavColorSecond = '';
	var $url = ''; // contient les variables
	var $url_var = ''; // contient les paramtres de l'url
	var $UrlRgxPatterns = ''; // contient le regex pour formatter les urls
	var $UrlRgxReplace = '';
	var $user_resulset = array(); // contient le resultat utilisateur

	// propritaire
	var $NbRecordPerPage = 0; // nombre de rsultats voulues par pages
	var $NbResults = 0; // les rsultats
	var $Count = 0;
	var $T_first = 0; // celui uliser pour dterminer l'Id
	var $First = 0;
	var $Last = 0;
	var $PageNumber = 0;
	var $PageCount = 0;
	var $NavColor = '';	

    var $OrderByFields = array();
    var $OrderByImgsPath = ''; // chemin par defaut des icone asc-desc

	// Error
	var $error_msg = '';
	var $error_signaled = array(); 
	
	// debugage
	var $struct_mode = 0;
	var $struct_tab = array(); // contient le tableau entier

	function loadPlugin($name)
	{
		/*
        $name = strtolower($name);

		$tmp = TPLN_PATH."/plugin/$name/$name.class.php";
		include_once($tmp);
		// PHP's version // cration dynmique d'objet
		$_php_ver = (float) PHP_VERSION;
		if($_php_ver < 5.0)
			aggregate($this, $name);
		else
		{
			die('PHP version invalid no support for PHP 5');
			// Cration dynamique d'objet sans la fonction aggregate
		}
        */
	}

	function HTMLCompress($bool)
	{
		$this->HTMLCompress = $bool;
	}

	function Dump($label = 'Debug', $data, $return = 0)
	{
		$format_html = 1;
		($format_html) ? $v = '<pre><div style="text-align:left;margin:5px;border:1px solid #CCCCCC;background-color:#E5E5E5;">' : $v = '<pre>';
		if(!empty($label))
		{
			if($format_html)$v .= '<div style="text-align:left;padding:5px;background-color:#CCCCCC;">';
			$v .= "<b>$label => </b> ";
			if($format_html)$v .= '</div>';
		}
		$v .= print_r($data, 1);
		($format_html) ? $v .= '</div></pre>' : $v .= '</pre>';

		if($return)
			return $v;
		else
			echo $v;

		return;
	}

	function getIP()
	{
		if(isset($_SERVER['HTTP_X_FORWARDED_FOR']))
			$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
		elseif(isset($_SERVER['HTTP_CLIENT_IP']))
			$ip = $_SERVER['HTTP_CLIENT_IP'];
		else
			$ip = $_SERVER['REMOTE_ADDR'];

		return $ip;
	}

	function FileStructMode()
	{
		$this->struct_mode = 1;
	}

	function TraceMode()
	{
		$this->trace_mode = 1;
	}

	// Parent function *******************************************************************************************
	function _StructOpen()
	{
		if($this->struct_mode == 0)
		{
			return;
		}

		$this->_StructFile();
		$this->_StructItems();
		$this->_StructBlocs();
		$this->_Arr2HtmlTab();
	}

	function _StructFile()
	{
		$this->struct_tab[] = "<b>File:</b> {$this->f[$this->f_no]['name']}<br>\n";
	}

	function _StructItems()
	{
		$p_var = array('_UrlBng',
			'_UrlPrev',
			'_UrlNext',
			'_UrlEnd',
			'_UrlPageNav',
			'_PageNumber',
			'_PageCount',
			'_First',
			'_Last',
			'_Count',
			'_NavColor',
			'_Chrono',
			'_Logo',
			'_Version',
			'_Field',
			'_QueryCount');

		$tab = '<b>Variable(s) found:</b> '.count($this->f[$this->f_no]['items'])."<br>\n";

		$cur_item = array(); // initiamisation
		$tpln_var = array(); // initialisation
		if(count($this->f[$this->f_no]['items']) > 0)
		{
			$item_tmp = array_unique($this->f[$this->f_no]['items']);

			foreach($item_tmp as $name)
			{
				if(!in_array($name, $p_var))
				{
					$cur_item[] = $name;
				}
				else
				{
					$tpln_var[] = $name;
				}
			}
		}

		$tab .= $this->_Arr2List('Constants variable(s)', $this->f[$this->f_no]['php_items'], '{CONST::', '}');
		$tab .= $this->_Arr2List('Php variable(s)', $this->f[$this->f_no]['php_items'], '{$', '}');
		$tab .= $this->_Arr2List('User variable(s)', $cur_item, '{', '}');
		$tab .= $this->_Arr2List('Private variable(s)', $tpln_var, '{', '}');
		$tab .= $this->_Arr2List('Tpln Include Command(s)', $this->f[$this->f_no]['cmd_items'], '{#include(', ')}');

		$this->struct_tab[] = $tab;
	}

	function _StructBlocs()
	{
		$this->struct_tab[] = $this->_Arr2List('Blocs', $this->f[$this->f_no]['shortcut_blocs']['all']);
	}

	// extra functions *************************************************************************************
	function _Arr2List($text = '', $arr, $bng = '', $end = '')
	{
		$txt = null;

		if(!empty($text))
			$txt .= "<b>$text:</b> ".count($arr).'<br>';

		if(count($arr) > 0)
		{
			$txt .= '<ul>';

			foreach($arr as $name)
			{
				$txt .= '<li>'.$bng.$name.$end."</li>\n";

				if($text == 'Blocs')
				{
					if(count($this->f[$this->f_no]['shortcut_blocs'][$name]['items']) > 0)
					{
						$txt .= '<ul>';

						foreach($this->f[$this->f_no]['shortcut_blocs'][$name]['items'] as $item)
						$txt .= "<li>\{$item}</li>";

						$txt .= '</ul>';
					}
				}
			}

			$txt .= '</ul>';
		}

		return $txt;
	}

	function _Arr2HtmlTab()
	{
		for($i = 0;$i < count($this->struct_tab);$i++)
		{
			if($i == 0)
				echo '<table width="100%" border="1" cellspacing="0" cellpadding="3">';

			echo "<tr><td>{$this->struct_tab[$i]}</td></tr>";

			if($i == count($this->struct_tab)-1)
				echo '</table>';
		}
	}
	
	// DataBase *******************************************************************************************************************/
	function DbConnect($db_type = '', $host = '', $login = '', $password = '', $base = '', $port = '', $new_connection = '')
	{
		// initialisation des variable
		if(empty($db_type))$db_type = TPLN_DB_TYPE_DEFAULT;
		if(empty($host))$host = TPLN_DB_HOST_DEFAULT;
		if(empty($login))$login = TPLN_DB_LOGIN_DEFAULT;
		if(empty($password))$password = TPLN_DB_PASSWORD_DEFAULT;
		if(empty($base))$base = TPLN_DB_BASE_DEFAULT;
		if(empty($port))$port = TPLN_DB_PORT;
		if(empty($new_connection))$new_connection = TPLN_DB_NEW_CONNECTION;

		// avec ou sans porc ;-) ?
		// $dsn = "mysql://$user:$pass@$host/$db_name";
		/* if(!empty($port))
			$dsn = "$db_type://$login:$password@$host:$port/$base";
		else
			$dsn = "$db_type://$login:$password@$host/$base";  */

		$this->db_index++;
        $this->db[$this->db_index] = ADONewConnection($db_type);
       	if(!empty($port))$this->db[$this->db_index]->port = $port;
				
        if($new_connection)
			$conn_res = @$this->db[$this->db_index]->NConnect($host, $login, $password, $base);
		else
			$conn_res = @$this->db[$this->db_index]->Connect($host, $login, $password, $base); 

		if(!$conn_res)
			$this->_DBError(0, $this->db[$this->db_index]->ErrorMsg());
	}

	function ChangeConnection($db_index)
	{
		if($db_index < 0 || $db_index >= count($this->db))
		{
			$this->_DBError('2.1', $db_index);
			return;
		}

		$this->db_index = $db_index;
	}
	
	function setConnectionID($db_index)
	{
		$this->ChangeConnection($db_index);
	}
	
	function getConnectionID()
	{
		return $this->db_index;
	}
	

	function ChangeQuery($req_index)
	{
		if($req_index < 0 || $req_index >= count($this->req))
		{
			$this->_DBError('2.11', $req_index);
			return;
		}

		$this->req_index = $req_index;
	}
	
	function SetQueryID($req_index)
	{
		$this->ChangeQuery($req_index);
	}
	
	function GetQueryNb()
	{
		return $this->req_index;
	}
	
	function GetQueryID()
	{
		return $this->GetQueryNb();
	}
	

	function DbClose()
	{
		if(!is_object($this->db[$this->db_index]))
		{
			$this->_DBError(0.1);
			return;
		}

		$this->db[$this->db_index]->Close();
		if(!$this->db[$this->db_index])
		{
			$this->_DBError(1, $this->db[$this->db_index]->getMessage());
		}
	}

	function checkQuotes($value)
	{
		if(is_array($value))
		{
			foreach($value as $key => $val)
			$value[$key] = $this->checkQuotes($val);

			return $value;
		}
		else
			return mysql_real_escape_string($value);
			
			//return ((!get_magic_quotes_runtime() && !get_magic_quotes_gpc()) ? mysql_real_escape_string($value) : $value);
			
	}
	function stripQuotes($value)
	{
		if(is_array($value))
		{
			foreach($value as $key => $val)
			$value[$key] = $this->stripQuotes($val);

			return $value;
		}
		else
			return stripslashes($value);
	}

	function GetMaxID($table, $ID)
	{
		$sql = "SELECT MAX($ID) FROM $table";
		$this->DoQuery($sql);
		return $this->getOne();
	}

	function DoQuery($query, $fetch_mode = ADODB_FETCH_ASSOC)
	{
        $GLOBALS['ADODB_FETCH_MODE'] = $fetch_mode;
        if(!is_object($this->db[$this->db_index]))
		{
			$this->_DBError(0.1);
			return;
		}

		$this->query_count++;
		$this->req_index++;
		$this->req[$this->req_index] = $this->db[$this->db_index]->Execute($query);
		if (!$this->req[$this->req_index])
		{ 
			// sans debuggage ?
			if(TPLN_SQL_QUERY_DEBUG == 0)
				$msg = $this->db[$this->db_index]->ErrorMsg();
			else
				$msg = $this->db[$this->db_index]->ErrorMsg()." <br><br>\n<pre><i><strong>".$query.'</strong></i></pre>';

			$this->DBLastError = $msg;
			$this->_DBError(2, $msg , TPLN_SQL_QUERY_DEBUG);
			return false;
		}

		return true;
	}

    function DBInsert($table, $arr, $exlude_fields=array())
    {
        $fields = array();
        $vals = array();
        foreach($arr as $key => $val)
        {
            // jokers
			$joker = false;
			foreach($exlude_fields as $ef)
			{
				$tmp = explode('*', $ef);
				if(count($tmp) == 2)
				{
					$str = $tmp[0];
					if(eregi("^$str", $key))
					{
						$joker = true;
						break;
					}
				}				
			}
						
			
			if(!in_array($key, $exlude_fields) && !$joker)
            {
                $fields[] = $key;
                $vals[] = $val;
            }
			
						
        }

        $sql = "INSERT INTO $table\n ";
        $sql .= "\t(\n\t\t".join(",\n\t\t", $fields)."\n\t)\n";
        $sql .= "VALUES\n";
        $sql .= "\t(\n";
        for($i=0; $i < count($vals); $i++)
        {
			if(!isset($vals[$i]))
                $s = 'NULL';
			elseif(is_int($vals[$i]) || is_float($vals[$i]) || in_array($vals[$i], array('NOW()', 'NULL')))
                $s = $vals[$i];
            else
                $s = "'".mysql_real_escape_string($vals[$i])."'";
            if($i < count($vals)-1)$s .= ',';
            $sql .= " \t\t$s\n";
        }
        $sql .= "\t)";
        return $this->DoQuery($sql);
    }
    
    function DBUpdate($table, $arr, $where='', $exlude_fields=array())
    {
        $fields = array();
        $vals = array();
        foreach($arr as $key => $val)
        {
            // jokers
			$joker = false;
			foreach($exlude_fields as $ef)
			{
				$tmp = explode('*', $ef);
				if(count($tmp) == 2)
				{
					$str = $tmp[0];
					if(eregi("^$str", $key))
					{
						$joker = true;
						break;
					}
				}				
			}			
			
			if(!in_array($key, $exlude_fields) && !$joker)
            {
                $fields[] = $key;
                $vals[] = $val;
            }
        }

        $sql = "UPDATE $table SET\n";
        for($i=0; $i < count($vals); $i++)
        {
			if(!isset($vals[$i]))
                $s = 'NULL';
            elseif(is_int($vals[$i]) || is_float($vals[$i]) || in_array($vals[$i], array('NOW()', 'NULL')))
                $s = $vals[$i];
            else
                $s = "'".mysql_real_escape_string($vals[$i])."'";
            if($i < count($vals)-1)$s .= ',';
            $sql .= " \t\t{$fields[$i]} = $s\n";
        }

        if(!empty($where))
            $where = "WHERE\n\t\t".$where;
        $sql .= $where;
		
        return $this->DoQuery($sql);
    }

    function DBDelete($table, $where='')
    {
        if(!empty($where))
            $where = 'WHERE '.$where;

        $sql = "DELETE FROM $table $where";
        
        return $this->DoQuery($sql);
    }

	function DBGetError()
	{
		return $this->DBLastError;
	}

	function GetRowsCount()
	{
		if(!is_object($this->req[$this->req_index]))
		{
			$this->_DBError(2.2);
			return;
		}

		$rows_count = $this->req[$this->req_index]->RecordCount();
		return $rows_count;
	}

	function DBNumRows()
	{
		return $this->GetRowsCount();
	}

    function  DBFetch()
    {
        if(!is_object($this->req[$this->req_index]))
		{
			$this->_DBError(2.2);
			return;
		}
		$row = $this->req[$this->req_index]->fetchRow();		
		return $row;
    }

	function DBFetchArray()
	{
		if(!is_object($this->req[$this->req_index]))
		{
			$this->_DBError(2.2);
			return;
		}

		$row = $this->req[$this->req_index]->fetchRow();
		return $row;
	}

	function DBFetchAssoc()
	{
		if(!is_object($this->req[$this->req_index]))
		{
			$this->_DBError(2.2);
			return;
		}
		$row = $this->req[$this->req_index]->fetchRow();
		return $row;
	}

	function DBFreeResult()
	{
		if(!is_object($this->req[$this->req_index]))
		{
			$this->_DBError(2.2);
			return;
		}

		$this->req[$this->req_index]->Close();
	}
	
	// prise des rsulats dans un array
	function GetData()
	{
		$i = 0;
        $results = array();
		while($row = $this->DBFetchAssoc())
			$results[$i++] = $row;

		return $results;
	}

	function GetOne()
	{
		$res = $this->GetData();

        if(count($res) == 0)
            return '';

        if(isset($res[0][0]))
            return $res[0][0];
        else
            foreach($res[0] as $key => $val)
                return $val;

	}

	function GetNumFields()
	{
		if(!is_object($this->req[$this->req_index]))
		{
			$this->_DBError(2.2);
			return;
		}

		$fields_count = $this->req[$this->req_index]->_numOfFields;
		return $fields_count;
	}

	function GetFields()
	{
		return array_keys($this->req[$this->req_index]->fields);
	}

	function GetFieldName($field_no)
	{
		if(!is_object($this->req[$this->req_index]))
		{
			$this->_DBError(2.2);
			return;
		}

		if($field_no < 0 || $field_no >= $this->GetNumFields())
		{
			$this->_DBError(2.3, $field_no);
			return;
		}
		
		// on prend la premire ligne
		// $row = $this->DBFetchAssoc();
		$row =  $this->req[$this->req_index]->fields;
		$col = array_keys($row);

		return $col[$field_no];
	}

	function GetDBList()
	{
		if(!is_object($this->db[$this->db_index]))
		{
			$this->_DBError(0.1);
			return;
		}
        $this->DoQuery('SHOW DATABASES', ADODB_FETCH_NUM);
		$db_list = $this->getData();

        $dbs = array();
        for($i=0;  $i < count($db_list) ; $i++)
            $dbs[] = $db_list[$i][0];

		return $dbs;
	}

	function GetTableList()
	{
		if(!is_object($this->db[$this->db_index]))
		{
			$this->_DBError(0.1);
			return;
		}

		$this->DoQuery('SHOW TABLES', ADODB_FETCH_NUM);

		while($row = $this->DbFetchArray())
		    $res[] = $row;

		return $res;
	}
	
	// constuction de la requete
	function _SetQuery($query)
	{ 
		// encodage de la requete
		// $query = str_replace("\n",' ',$query);
		// $query = str_replace("\r",' ',$query);		
		
		/*if(count($this->cons_query) > 0 || is_array($this->user_resulset)) return;

		$this->cons_query['STRING'] = $query; 
		// on parcours la requete  l'envers ;-)'
		// limit ?
		if(preg_match("/LIMIT(.*)/msi", $query, $match))
		{
			$this->cons_query['LIMIT'] = trim($match[1]);
			$query = str_replace($match[0], '', $query); // on remplace ds la chaine
		}
		// order by
		if(preg_match("/ORDER BY(.*)/msi", $query, $match))
		{
			$this->cons_query['ORDER BY'] = trim($match[1]);
			$query = str_replace($match[0], '', $query); // on remplace ds la chaine

            // y a t il une clause speciale ? order by filter ?
            if(isset($_GET['torder_by']))
                $this->cons_query['ORDER BY'] = "{$_GET['torder_by']} {$_GET['tsens']}, ".$this->cons_query['ORDER BY'];
		}
        else
        {
            if(isset($_GET['torder_by']))
                $this->cons_query['ORDER BY'] = "{$_GET['torder_by']} {$_GET['tsens']}";
        }


        // group by
		if(preg_match("/GROUP BY(.*)/msi", $query, $match))
		{
			$this->cons_query['GROUP BY'] = trim($match[1]);
			$query = str_replace($match[0], '', $query); // on remplace ds la chaine
		}
		// where
		if(preg_match("/WHERE(.*)/msi", $query, $match))
		{
			$this->cons_query['WHERE'] = trim($match[1]);
			$query = str_replace($match[0], '', $query); // on remplace ds la chaine
		}
		// from
		if(!preg_match("/FROM(.*)/msi", $query, $match))$this->_DBError(4);

		$this->cons_query['FROM'] = trim($match[1]);
		$query = str_replace($match[0], '', $query); // on remplace ds la chaine
		// select
		if(!preg_match("/SELECT(.*)/msi", $query, $match))$this->_DBError(3);

		$this->cons_query['SELECT'] = trim($match[1]);
		$query = str_replace($match[0], '', $query); // on remplace ds la chaine
		*/
	}

	// recr une requete avec les nouveau paramtres de LIMIT ;-)
	function _SetQueryLimit()
	{
		if(empty($this->PageNumber))
		{
			$this->PageNumber = 1;
		}
		// calcul du nombre de pages
		if($this->NbRecordPerPage == 0)
		{
			$this->PageCount = 1;
			$this->PageNumber = 1;
			$this->First = 1;
			$this->T_first = $this->First;
			$this->Last = $this->Count;
		}
		else
		{
			$this->PageCount = ceil($this->Count / $this->NbRecordPerPage); // arrondi a l'entier superieur
			if($this->PageNumber > $this->PageCount)
			{
				$this->PageNumber = $this->PageCount;
			}

			// on determine debut du limit
			$this->First = ($this->PageNumber - 1) * $this->NbRecordPerPage;
			$this->T_first = $this->First ;
			$this->Last = $this->First + $this->NbRecordPerPage;
		}

		/* v2.9
		if(!is_array($this->user_resulset))
		{
			// on reconstruit la requete
			$query = "SELECT \n";
			    $query .= $this->cons_query['SELECT']." \n";
			$query .= " FROM \n";
			$query .= $this->cons_query['FROM']." \n";
			// Where
			if(!empty($this->cons_query['WHERE']))
			{
				$query .= ' WHERE '." \n";
				$query .= $this->cons_query['WHERE']." \n";
			}
            // Group By
			if(!empty($this->cons_query['GROUP BY']))
			{
				$query .= ' GROUP BY '." \n";
				$query .= $this->cons_query['GROUP BY']." \n";
			}
			// Order By
			if(!empty($this->cons_query['ORDER BY']))
			{
				$query .= ' ORDER BY '." \n";;
				$query .= $this->cons_query['ORDER BY']." \n";
			}
		}
		*/

		// limit ?
		if($this->NbRecordPerPage > 0)
		{			
			// if($this->First == 0){$this->First = 1;}
			$this->First++;
			if($this->Last > $this->Count)
			{
				$this->Last = $this->Count;
			}

			/* v2.9
			if(!is_array($this->user_resulset))
				$query .= " LIMIT  $this->First,$this->NbRecordPerPage";
			*/			
		}

		/* v2.9
		if(!is_array($this->user_resulset))
			return $query;
		*/
	}

	// retourne la bonne url formatte
	function _getUrl($t_pg = '')
	{
		$url = $this->Url."tpg=$t_pg".$this->url_var;

		if(!empty($this->UrlRgxPatterns) && !empty($this->UrlRgxReplace))
			$url = @preg_replace('/'.$this->UrlRgxPatterns.'/i', $this->UrlRgxReplace, $url);

		return $url;
	}

	function RewriteUrl($patterns, $replace)
	{
		$this->UrlRgxPatterns = $patterns;
		$this->UrlRgxReplace = $replace;
	}

	function SetNavColor($color1, $color2)
	{
		$this->NavColorFirst = $color1;
		$this->NavColorSecond = $color2;
	}

	// prise totale des rsulats
	function _GetTotalCount()
	{
		/* v2.9
		
		$query = 'SELECT ';

        // on regarde s'il y a un distinct
        $distinct = false;
        $d = trim($this->cons_query['SELECT']);
        if(strpos($d, 'DISTINCT') === false)
            $query .= ' COUNT(*)';
        else
        {
            $distinct = true;
            // on capture le champs qui possede le distinct
            list($f) = explode(',', $this->cons_query['SELECT']);
            list($f) = explode(' AS ', $f);
            $query .= " COUNT($f) ";
        }

		$query .= 'FROM ';
		$query .= $this->cons_query['FROM'];

		if(!empty($this->cons_query['WHERE']))
		{
			$query .= ' WHERE ';
			$query .= $this->cons_query['WHERE'];
		}

		$this->DoQuery($query) or die($this->DBGetError());

        // group by sans clause distinct
		if(eregi('group by', $query) && !$distinct)
			$this->Count = $this->GetRowsCount();
		else
			$this->Count = $this->GetOne();
		*/
	}


	function _ApplyPrivateVar()
	{
		if(empty($this->Url))
			$this->Url = basename($_SERVER['PHP_SELF']).'?';

		// Variable TPLN
		if($this->ItemExists('_Count', 'data'))
			$this->Parse('data._Count', $this->Count);

		if($this->ItemExists('_First', 'data'))
			$this->Parse('data._First', $this->First);

		if($this->ItemExists('_Last', 'data'))
			$this->Parse('data._Last', $this->Last);

		if($this->ItemExists('_PageCount', 'data'))
			$this->Parse('data._PageCount', $this->PageCount);

		if($this->ItemExists('_PageNumber', 'data'))
			$this->Parse('data._PageNumber', $this->PageNumber);

        // order by ?
        for($i=0; $i < count($this->OrderByFields); $i++)
        {
            $f = $this->OrderByFields[$i];
            $img = '';
            $tpg = (isset($_GET['tpg'])) ? $_GET['tpg'] : 1;
            $u = $this->_getUrl($tpg);
            if($_GET['torder_by'] == $f)
            {
                $sens = ($_GET['tsens'] == 'asc') ? 'desc' : 'asc';
                $u = str_replace("tsens={$_GET['tsens']}", "tsens=$sens", $u);
                $img = "<a href=\"$u\"><img src=\"$this->OrderByImgsPath/order_{$_GET['tsens']}_actived.gif\" style=\"border:0;\" align=\"absmiddle\" /></a>";
            }
            else
            {
                $u = str_replace("torder_by={$_GET['torder_by']}&tsens={$_GET['tsens']}", "torder_by=$f&tsens=desc", $u);
                $img = "<a href=\"$u\"><img src=\"$this->OrderByImgsPath/order_desc.gif\" style=\"border:0;\" /></a>";
                //$u = str_replace("torder_by={$_GET['torder_by']}&tsens={$_GET['tsens']}", "torder_by=$f&tsens=asc", $u);
                //$img .= "<a href=\"$u\"><img src=\"$this->OrderByImgsPath/order_asc.gif\" style=\"border:0;\" /></a>";
            }

            $this->Parse("data._order_by::$f", $img);
        }



		// PREVIOUS
		if($this->BlocExists('previous'))
		{
			if(!$this->ItemExists('_Url', 'previous'))
			{
				$this->_Error('1.1', $this->f[$this->f_no]['name'], 'previous', '_Url');
				return;
			}

			if($this->PageNumber <= $this->PageCount /*&& $this->PageCount != 1 && $this->PageNumber != 1*/)
			{
				$prev_pg = $this->PageNumber - 1;
				$url = $this->_getUrl($prev_pg);
				$this->Parse('data.previous._Url', $url);
			}
			else
				$this->EraseBloc('data.previous');
		}

		// NEXT
		if($this->BlocExists('next'))
		{
			if(!$this->ItemExists('_Url', 'next'))
			{
				$this->_Error('1.1', $this->f[$this->f_no]['name'], 'next', '_Url');
				return;
			}

			/*if($this->PageNumber < $this->PageCount /*&& $this->PageCount != 1)
			{
			*/	$next_pg = $this->PageNumber + 1;
				$url = $this->_getUrl($next_pg);
				$this->Parse('data.next._Url', $url);
			/*}
			else
				$this->EraseBloc('data.next');
			*/
		}

		// START
		if($this->BlocExists('start'))
		{
			if(!$this->ItemExists('_Url', 'start'))
			{
				$this->_Error('1.1', $this->f[$this->f_no]['name'], 'start', '_Url');
				return;
			}

			//if($this->PageCount > 1)
			//{
				$url = $this->_getUrl(1);
				$this->Parse('data.start._Url', $url);
			//}
			//else
				//$this->EraseBloc('data.start');
		}

		// END
		if($this->BlocExists('end'))
		{
			if(!$this->ItemExists('_Url', 'end'))
			{
				$this->_Error('1.1', $this->f[$this->f_no]['name'], 'end', '_Url');
				return;
			}

			//if($this->PageCount > 1)
			//{
				$url = $this->_getUrl($this->PageCount);
				$this->Parse('data.end._Url', $url);
			//}
			//else
				//$this->EraseBloc('data.end');
		}

		// PAGER
		if($this->BlocExists('pager'))
		{
			// bloc out & in
			if(!$this->BlocExists('out'))
			{
				$this->_Error(2, $this->f[$this->f_no]['name'], 'out');
				return;
			}
			if(!$this->ItemExists('_Url', 'out'))
			{
				$this->_Error('1.1', $this->f[$this->f_no]['name'], 'out', '_Url');
				return;
			}
			if(!$this->ItemExists('_Page', 'out'))
			{
				$this->_Error('1.1', $this->f[$this->f_no]['name'], 'out', '_Page');
				return;
			}

			if(!$this->BlocExists('in'))
			{
				$this->_Error(2, $this->f[$this->f_no]['name'], 'in');
				return;
			}

			if(!$this->ItemExists('_Page', 'in'))
			{
				$this->_Error('1.1', $this->f[$this->f_no]['name'], 'in', '_Page');
				return;
			}

			if($this->PageCount >= 1)
			{
				$in = $this->GetBlocInFile('data.pager.in');
				$out = $this->GetBlocInFile('data.pager.out');
				// $this->EraseBloc("in");
				// $this->EraseBloc("out");
				$str = '';

				for($l = 1; $l <= $this->PageCount; $l++)
				{
					if($l == $this->PageNumber)
					{
						$str .= str_replace('{_Page}', $l, $in);
					}
					else
					{
						$url = $this->_getUrl($l);

						$tmp = str_replace('{_Page}', $l, $out);
						$tmp = str_replace('{_Url}', $url, $tmp);

						$str .= $tmp;
					}
				}

				$this->ParseBloc('data.pager', $str);
			}
			else
			{
				$this->EraseBloc('data.pager');
			}
		}

		// initialise la navigation
		if(empty($this->NavColorFirst))
		{
			$this->NavColorFirst = TPLN_DB_NavColorFirst;
		}
		if(empty($this->NavColorSecond))
		{
			$this->NavColorSecond = TPLN_DB_NavColorSecond;
		}
	}

	function SetUrl($url)
	{
		$this->Url = $url;
	}

	function UrlAddVar($url_add)
	{
		$this->url_var = "&$url_add";
	}

	function CreateFieldVars($path, $fields)
	{
		// chemin dfini en array
		if(is_array($path))
		{
			for($i = 0;$i < count($path);$i++)
			{
				$this->CreateFieldVars($path[$i], $fields);
			}
			return;
		}

		$arr = @explode('.', $path);

		if(count($arr) == 1)
		{
			$this->_Error('13', $this->f[$this->f_no]['name'], $arr[0]);
			return;
		}

		$lastbloc = $arr[count($arr)-1];

		// Verification du mot clefs _Field
		if(!$this->ItemExists('_Field', $lastbloc) && !$this->ItemExists('_FieldLabel', $lastbloc))
		{
			$this->_Error('1.1', $this->f[$this->f_no]['name'], $lastbloc, '_Field');
			return;
		}

		$cur_bloc_ini = $this->GetBlocInFile($lastbloc);
		$str = '';
		$all = '';
		$tab = array();

		for($i = 0;$i < count($fields);$i++)
		{
			// on remplace le mot clef field
			$str = str_replace('{_Field}', '{'.$fields[$i].'}', $cur_bloc_ini);
			$str = str_replace('{_FieldId}', $i, $str);
			$str = str_replace('{_FieldLabel}', str_replace('_', ' ', ucfirst($fields[$i])), $str);
			$all .= $str;
		}
		// on remplace le contenu du champs
		// $this->ParseBloc($path, $all);
		$this->ParseBloc($lastbloc, $all);
		// on retire le dernier du path
		// on veut que le chemin des peres
		$path = $this->_GetFathers($path, 'ARRAY', 0);
		$path = array_slice ($path, 0, count($path)-1);
		$path = join($path, '.');

		$this->ReloadBlocVars($path);
	}

    function ShowRecordsOrderBy($fields, $img_dir='')
    {
        $this->OrderByFields = $fields;

        if(!empty($img_dir))
            $this->OrderByImgsPath = $img_dir;
        else
            $this->OrderByImgsPath = TPLN_WEB_PATH.'/img';

        if(!isset($_GET['torder_by']) || !in_array($_GET['torder_by'], $fields))
            $_GET['torder_by'] = $fields[0];

        if(!isset($_GET['tsens']) || !in_array($_GET['tsens'], array('asc','desc')))
            $_GET['tsens'] = 'desc';

        $this->url_var .= "&torder_by={$_GET['torder_by']}&tsens={$_GET['tsens']}";
    }

	function ShowRecords($query, $nb_result_per_page = 0, $func_data = '')
	{
		// vrification du second paramtre
		if(!is_int($nb_result_per_page))
		{
			$this->_DBError(5);
			return;
		}
		if($nb_result_per_page < 0)
		{
			$this->_DBError('5.1');
			return;
		}

		// verification de l'array de resulats
		$this->breaker_name = (!$this->BlocExists('breaker')) ? '' : 'breaker';
		$this->user_resulset = $query;

		$this->NbRecordPerPage = $nb_result_per_page;

		if($this->NbRecordPerPage != 0 && isset($_GET['tpg']))
			$this->PageNumber = $_GET['tpg']; // variable passe en parametre
		else
			$this->PageNumber = null;

		// sql version
		if(!is_array($this->user_resulset))
		{
            $this->_SetQuery(''); // reconstruit la requete et assigne les variables
			// $this->_GetTotalCount(); //  nb total d'enregistrements
			
			//$res =  $this->db[$this->db_index]->Execute($query);			
			// $this->Count = $res->numRows();
			// $this->query_count++;
			
			// y a t il une clause speciale ? order by filter ?
            if(isset($_GET['torder_by']) && in_array($_GET['torder_by'], $this->OrderByFields))
			{
				// une clause de fin ?
				if(preg_match("#(.*)ORDER BY(.*)$#msi", $query))
				{
					$query .= " ,{$_GET['torder_by']} {$_GET['tsens']}";
				}
				else
				{
					$query .= " ORDER BY {$_GET['torder_by']} {$_GET['tsens']}";
				}
			}
			
			$this->DoQuery($query);
			$this->Count = $this->DBNumRows();
															
			$this->_SetQueryLimit(); // requete contenant les limites
									
			$this->req_index++;
			$tmp_nb_result_per_page = $nb_result_per_page;
			if($tmp_nb_result_per_page == 0)$tmp_nb_result_per_page = -1;
			
			

			$this->req[$this->req_index] = @$this->db[$this->db_index]->SelectLimit($query, $tmp_nb_result_per_page, ($this->PageNumber-1) * $nb_result_per_page);
			
			// erreur ?
			if (!$this->req[$this->req_index])
			{ 
				// sans debuggage ?
				if(TPLN_SQL_QUERY_DEBUG == 0)
					$msg = $this->db[$this->db_index]->ErrorMsg();
				else
					$msg = $this->db[$this->db_index]->ErrorMsg()." <br><br>\n<pre><i><strong>".$query.'</strong></i></pre>';
				
				$this->DBLastError = $msg;
				$this->_DBError(2, $msg , TPLN_SQL_QUERY_DEBUG);
				return false;
			}			
			
			$this->query_count++;
			
        }
		else
		{
			$this->Count = count($this->user_resulset);
			$this->_SetQuery(''); // reconstruit la requete et assigne les variables
		}

		// pas du tout d'enregistrement
		if($this->Count == 0)
		{
			$no_records = $this->GetBlocInFile('data.norecord');
			$this->EraseBloc('data.norecord');
			$this->ParseBloc('data', $no_records);

			$this->cons_query = array();
			return $this->Count;
		}

		if(!is_array($this->user_resulset))
		{
			// $this->DoQuery($new_query, 2) or die($this->DBGetError()); // execute la requete

			// $results = $this->GetData(); // recupere les rsultats
			$this->NbResults = $this->DbNumRows(); // resultats obtenues != $this->count
		}
		else
		{
			$this->_SetQueryLimit(); // requete contenant les limites
			$this->NbResults = count($this->user_resulset);
		}

		if($this->NbResults == 0) // il y'en a pas pour une recherche
		{
			// a redefinir pour la recherche
			$no_records = $this->GetBlocInFile('data.norecord');
			$this->EraseBloc('data.norecord');
			$this->ParseBloc('data', $no_records);

			$this->cons_query = array();
			return $this->Count;
		}
		else // si il y a des rsultats alors
		{
			$this->EraseBloc('data.norecord'); // on efface norecord
			$this->_ApplyPrivateVar(); // on modifie les items propriaitaire $Count...
			if(!is_array($this->user_resulset) && count($this->f[$this->f_no]['shortcut_blocs']['loop']['items']) == 0)
			{
				$this->cons_query = array();
				return $this->Count;
			}
			// parsing
			$this->T_i = 0; // pour la _NavColor
			$this->T_id = $this->T_first; // pour _Id
			
			if($this->T_id == 0)$this->T_id = 1;
			// $this->T_id++;
			
			$this->last_header = '';
			if(!is_array($this->user_resulset))
			{				
				while($row = $this->DbFetchAssoc())
				{
					$this->_trtShowRecords($row, $func_data);
				}
			}
			else
			{ 
				// on prend uniquement les resultats dsirs
				if($this->PageCount > 1)
				{
					$this->user_resulset = array_slice($this->user_resulset, $this->First-1, $this->NbRecordPerPage);
				}

				foreach($this->user_resulset as $row)
				{
					$this->_trtShowRecords($row, $func_data);
				}
			}
		}
		
		$this->Loop('data');
		
		$this->cons_query = array();
		return $this->Count;
	}
	
	var $T_i = 0;
	var $T_id = 0;
	var $last_header;
	var $breaker_name;
	var $header_count;
	function _trtShowRecords($row, $func_data)
	{		
		// if(!empty($func_data))$row = $func_data($row);
		if(!empty($func_data))
		{
			// methode ?
			if(strpos($func_data, '::') !== false || strpos($func_data, '->') !== false) // methode ?
				eval("\$row = $func_data(\$row);");
			else
				$row = $func_data($row);   
		}

		// breaker
		if(!empty($this->breaker_name) && $this->last_header != $row['breaker'])		
		{			
			$this->last_header = $row['breaker'];
			$this->header_count = 0;
		}

		$keys = @array_keys($row); // prise du mon des clefs
		foreach($keys as $key)
		{ 			
			// ajout de la nav color
			if($this->ItemExists('_NavColor', 'loop'))
			{
				$color = ($this->T_i % 2) ? $this->NavColorFirst : $this->NavColorSecond;
				$this->Parse('data.loop._NavColor', $color);
			}

			// ajout de l'Id
			if($this->ItemExists('_Id', 'loop'))
				$this->Parse('data.loop._Id', $this->T_id);

			if(!is_int($key) && $this->ItemExists($key, 'loop'))
				$this->Parse("data.loop.$key", $row[$key]);

			// breaker
			if(!empty($this->breaker_name))
			{
				if(!is_int($key) && $this->ItemExists($key, 'breaker'))
				{
					$this->Parse("data.loop.breaker.$key", $row[$key]);
				}
			}
		}
		
		$this->T_i++;
		$this->T_id++;
		
		// breaker
		if(!empty($this->breaker_name))
		{
			if($this->header_count == 1)
			{
				$this->EraseBloc('data.loop.breaker');
				$this->header_count = 0;
			}
			
			// $this->Loop('data.loop.breaker');
			$this->header_count++;
		}

		
		$this->Loop('data.loop');
	}
	
	
	// Gestion des erreurs
	function _Error($err_no, $file = '', $bloc = '', $item = '')
	{
		global $_err;
		$err_msg = $_err["$err_no"];
		$err_msg = str_replace(
								array('[:FILE:]', '[:BLOC:]', '[:ITEM:]'),
								array($file, $bloc, $item),
								$err_msg);

		$this->error_msg = "<B>TPLN Error $err_no:</B> $err_msg";
		$this->_OutPutMessage();
	}

	function _DBError($err_no, $msg = '', $exit = 1)
	{
		global $_err;

		$err_msg = $_err['DB']["$err_no"];
		$err_msg = str_replace('[:MSG:]', $msg, $err_msg);
		$this->error_msg = "<B>TPLN DB Error $err_no:</B> <table border=1><tr><td>$err_msg</td></tr></table>";
		$this->_OutPutMessage($exit);
	}
	
	// ben alert par email de l'admin
	function _MailAlert()
	{
		$err_alert = TPLN_ERROR_ALERT;
		$mail_admin = TPLN_MAIL_ADMIN;

		if(($err_alert == 1 && !empty($mail_admin)) && (!isset($_GET['tpln_w']) || $_GET['tpln_w'] != 'adm'))
		{
			$request_url_simple = str_replace('?'.$_SERVER['QUERY_STRING'], '', $_SERVER['REQUEST_URI']);
			$url = 'http://'.$_SERVER['HTTP_HOST'].$request_url_simple; 
			
			// possede une query string ?
			if(empty($_SERVER['QUERY_STRING']))
				$url .= '?tpln_w=adm';
			else
				$url .= '?'.$_SERVER['QUERY_STRING'].'&tpln_w=adm';

			$subject = '[TPLN] Alert Error';

			$err_msg = $this->error_msg;
			$err_msg = str_replace('&lt;', '<', $err_msg);
			$err_msg = str_replace('&gt;', '>', $err_msg);

			$body = date('[Y-m-d H:i] ')." TPLN has detected an error\n\n";
			$body .= $err_msg.' in '.$_SERVER['SCRIPT_FILENAME']."\n\n\n";
			$body .= "Url <a href=\"$url\">$url</a>\n";
			$body .= "===========================================\n";
			$body .= 'TPLN version '.TPLN_VERSION."\n";
			$body .= 'IP: '.$this->GetIP()."\n";

			$body = str_replace("\n", "<br />", $body);

			$headers = "MIME-Version: 1.0\n";
			$headers .= "Content-type: text/html; charset=iso-8859-1\n";
			$headers .= "From: ".TPLN_MAIL_EXPEDITOR."\n";
			$headers .= 'X-Mailer: PHP/'.phpversion();

			if(@mail(TPLN_MAIL_ADMIN, $subject, $body, $headers))
			{
				if(TPLN_LANG == 'en')
				{
					$msg = "<br><br><hr>An email has been sent to the webmaster <a href=\"mailto:$mail_admin\">$mail_admin</a>";
				}
				if(TPLN_LANG == 'fr')
				{
					$msg = "<br><br><hr>Un Email a t envoy au webmaster <a href=\"mailto:$mail_admin\">$mail_admin</a>";
				}

				$this->error_msg .= $msg;
			}
		}
	}
	
	// pour la sortie du message
	function _OutPutMessage($exit = true)
	{
		$this->_MailAlert();

		if(!@in_array($this->error_msg, $this->error_signaled))
		{
			if($exit == 1 && TPLN_ERROR_URI == '')
			{
				echo $this->error_msg;
				echo "<br/>\n";
			}

			$this->error_signaled[] = $this->error_msg;
		}
		
		// logs des erreurs
		if(TPLN_ERROR_LOGS)
		{ 
			// on regarde si le fichier existe sinon on le cre
			if(!($fp = @fopen(TPLN_ERROR_LOGS_FILE , 'a+')))
			{
				echo 'Impossible to open/create '.TPLN_ERROR_LOGS_FILE;
			}
			else
			{ 
				// jour heure,msg
				$txt = date('d-m-j H:i,').strip_tags($this->error_msg)."\n";
				fwrite($fp, $txt);
				fclose($fp);
			}
		}
		
		// sortie ?
		if(TPLN_ERROR_URI == '')
		{
			if($this->struct_mode == 0 && $exit == 1)
				exit();
		}
		else
		{
			$this->redirect(TPLN_ERROR_URI);
		}

	}
	
	function redirect($uri)
	{
		if(!headers_sent())
		{
			header('Location: '.$uri);
			exit();
		}
		else
		{			
			die('<script>document.location.href="'.$uri.'"</script>');
		}	
	}
	
	
	
	// fonction constructrice de classe
	function TPLN()
	{ 
		// regarde si le repertoire par dfaut existe
		if(TPLN_DEFAULT_IND == 1)
		{
			if(!is_dir(TPLN_DEFAULT_PATH))
			{
				if(!@mkdir(TPLN_DEFAULT_PATH, 0755))
				{
					$this->_Error(7.2);
					return;
				}

				clearstatcache();
			}
		}
	}

	
	function _initTemplateVars()
	{
		// initialisation des types de variables pour EALL **************************
		$this->f[$this->f_no]['name'] = null;
		$this->f[$this->f_no]['cache_name'] = null;
		$this->f[$this->f_no]['buffer'] = null;
		$this->f[$this->f_no]['items'] = array();
		$this->f[$this->f_no]['constant_items'] = array();
		$this->f[$this->f_no]['php_items'] = array();
		$this->f[$this->f_no]['cmd_items'] = array();

		$this->f[$this->f_no]['create_cached_file'] = 0;
		$this->f[$this->f_no]['time_started'] = 0;
		$this->f[$this->f_no]['cache_expire'] = 0;
		$this->f[$this->f_no]['execution_time'] = 0;
		$this->f[$this->f_no]['chrono_started'] = 0;

		$this->f[$this->f_no]['shortcut_blocs'] = array();
		$this->f[$this->f_no]['shortcut_blocs']['all'] = array();
		$this->f[$this->f_no]['shortcut_blocs']['used'] = array();

		$this->f[$this->f_no]['shortcut_blocs']['name'] = null;
		$this->f[$this->f_no]['shortcut_blocs']['name']['items'] = array();

		$this->f[$this->f_no]['def_blocs'] = array();

		$this->chrono_type[$this->f_no] = null;
	}

	function Open($file = '', $cached = '', $cached_time = '')
	{
		$this->f_no++; // incremente
		$this->_initTemplateVars(); // initialisation de notre tableau

		// ajout du nom de fichiers
		if(empty($file))
		{
			$file = basename($_SERVER['PHP_SELF']);
			// on prend l'extension et on l'efface
			preg_match("/\.([^\.]*$)/", $file, $elts);

			if(count($elts) > 1)
				$file = str_replace($elts[count($elts)-1], TPLN_DEFAULT_EXT, $file);
		}

		// verifie l'existence de l'extension et du rpertoire par dfault
		if(TPLN_DEFAULT_IND)
		{
			if(!preg_match("/\./", basename($file)))
			{
				$file .= '.'.TPLN_DEFAULT_EXT;
			}// met l'extension par defaut s'il ni en a pas
			$file = TPLN_DEFAULT_PATH.'/'.$file; // met le repertoire par defaut
		}

		$this->f[$this->f_no]['name'] = $file;

		$time = explode(' ', microtime());
		$this->f[$this->f_no]['time_started'] = time(); // dans pour le cache  la seconde
		$this->f[$this->f_no]['chrono_started'] = $time[1] + $time[0];

		// on dfinit le chrono
		if(!defined('TPLN_CHRONO_STARTED'))
		{
			$chrono = $time[1] + $time[0];
			define('TPLN_CHRONO_STARTED', $chrono);
		}

		// on regarde si le fichier est en cache
		if(!empty($cached))
		{
			// le ? est remplac par -- dans le fichier en cache
			if(strpos($file, '?'))
				$this->f[$this->f_no]['name'] = substr($this->f[$this->f_no]['name'], 0, strpos($file, '?'));

			$this->f[$this->f_no]['cache_name'] = str_replace('?', '---', $file);
			$this->_CacheDirExists();

			if(empty($cached_time))
			{
				$cached_time = TPLN_CACHE_TIME;
			}
			$this->f[$this->f_no]['cache_expire'] = $this->f[$this->f_no]['time_started'] + $cached_time; // cache d'expiration

			// on regarde le fichier est encore en cache
			// fichier existe && sa date de creation est <= au temps de cache
			if(file_exists(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['cache_name']) && $this->_InCachePeriod())
			{
				$this->_GetCachedFile();
				return true;
			}
			else
			{
				$this->f[$this->f_no]['create_cached_file'] = 1;
				$this->f[$this->f_no]['buffer'] = $this->GetFile($this->f[$this->f_no]['name']);
			}
		}
		else
			$this->f[$this->f_no]['buffer'] = $this->GetFile($this->f[$this->f_no]['name']);

		// remplace les constantes
		$this->ParseConstants();

        $this->_ParsePhpCommands();

		// remplace les variables avec $
		if(TPLN_PARSE_GLOBALS)$this->ParseGlobals();
		// on remplace les fichiers  inclure
		$this->_CaptureIncludeCmd(); // capture des commandes include
		// parsing et valuation si necessaire
		if(count($this->f[$this->f_no]['cmd_items']) > 0)
		{
			$this->_ParseAllIncludeCmd();
			$this->f[$this->f_no]['cmd_items'] = array(); // on efface
		}
		// on reremplace les $ si contenues dans fichiers inclu
		if(TPLN_PARSE_GLOBALS)$this->ParseGlobals();



		$this->f[$this->f_no]['shortcut_blocs']['all'] = $this->_CaptureAllBlocs();
		$this->_EndBlocVerify();
		$this->_DualBlocVerify();

		// Capture des items apres verification des blocs
		$this->f[$this->f_no]['items'] = $this->_CaptureItems();
		$this->_CaptureItemsInEachBloc();

		// debugger
		$this->_StructOpen();

		if($this->f[$this->f_no]['create_cached_file'] == 1)return false;
	}

	function ParseConstants()
	{
		$this->f[$this->f_no]['constant_items'] = $this->_CaptureItems('', 'CONSTANT');
		if(count($this->f[$this->f_no]['constant_items']) == 0)return;

		$defined_constants = get_defined_constants();

		foreach($this->f[$this->f_no]['constant_items'] as $item)
		{
			$this->f[$this->f_no]['buffer'] = $this->_ReplaceItem('CONST::'.$item, $defined_constants[$item], $this->f[$this->f_no]['buffer']);
		}
	}

	function CleanBlocs()
	{
		for($i = 0; $i < count($this->f[$this->f_no]['shortcut_blocs']['all']); $i++)
		{
			$this->f[$this->f_no]['buffer'] = str_replace(
				array("<bloc::{$this->f[$this->f_no]['shortcut_blocs']['all'][$i]}>",
					"</bloc::{$this->f[$this->f_no]['shortcut_blocs']['all'][$i]}>"
					) , '', $this->f[$this->f_no]['buffer']); // bloc avant/apres
		}
	}

	function CreateVirtualTemplate($countain)
	{
		$this->vf_no++; // incremente
		$this->f_no++; // incremente

		// initialisation des types de variables pour EALL **************************
		$this->f[$this->f_no]['name'] = 'Virtual'.$this->vf_no;
		$this->f[$this->f_no]['buffer'] = $countain;
		$this->f[$this->f_no]['items'] = array();

		$this->f[$this->f_no]['constant_items'] = array();
		$this->f[$this->f_no]['php_items'] = array();
		$this->f[$this->f_no]['cmd_items'] = array();

		$this->f[$this->f_no]['create_cached_file'] = 0;
		$this->f[$this->f_no]['time_started'] = 0;
		$this->f[$this->f_no]['cache_expire'] = 0;
		$this->f[$this->f_no]['execution_time'] = 0;
		$this->f[$this->f_no]['chrono_started'] = 0;

		$this->f[$this->f_no]['shortcut_blocs'] = array();
		$this->f[$this->f_no]['shortcut_blocs']['all'] = array();
		$this->f[$this->f_no]['shortcut_blocs']['used'] = array();

		$this->f[$this->f_no]['shortcut_blocs']['name'] = null;
		$this->f[$this->f_no]['shortcut_blocs']['name']['items'] = array();

		$this->f[$this->f_no]['def_blocs'] = array();

		$this->chrono_type[$this->f_no] = null;

		$time = explode(' ', microtime());
		$this->f[$this->f_no]['time_started'] = time(); // dans pour le cache  la seconde
		$this->f[$this->f_no]['chrono_started'] = $time[1] + $time[0];

		// on dfinit le chrono
		if(!defined('TPLN_CHRONO_STARTED'))
		{
			$chrono = $time[1] + $time[0];
			define('TPLN_CHRONO_STARTED', $chrono);
		}

		$this->ParseConstants();

		// remplace les variables avec $
		if(TPLN_PARSE_GLOBALS)$this->ParseGlobals();

		// on remplace les fichiers  inclure
		$this->_CaptureIncludeCmd(); // capture des commandes include

		// parsing et valuation si necessaire
		if(count($this->f[$this->f_no]['cmd_items']) > 0)
		{
			$this->_ParseAllIncludeCmd();
			$this->f[$this->f_no]['cmd_items'] = array(); // on efface
		}
		// on reremplace les $ si contenues dans fichiers inclu
		if(TPLN_PARSE_GLOBALS) $this->ParseGlobals();

		$this->f[$this->f_no]['shortcut_blocs']['all'] = $this->_CaptureAllBlocs();
		$this->_EndBlocVerify();
		$this->_DualBlocVerify();

		// Capture des items apres verification des blocs
		$this->f[$this->f_no]['items'] = $this->_CaptureItems();
		$this->_CaptureItemsInEachBloc();

		// debugger
		$this->_StructOpen();

		if($this->f[$this->f_no]['create_cached_file'] == 1)
			return false;
	}

	function DirectIOWrite($file = '')
	{
		$c_id = $this->f_no; // on sauve le numero actuel
		$this->Open($file);
		$this->Write();
		$this->f_no = $c_id; // on remet le numero
	}

	function DirectIOOutput($file = '')
	{
		$c_id = $this->f_no; // on sauve le numero actuel
		$this->Open($file);
		$output = $this->OutPut();
		$this->f_no = $c_id; // on remet le numero
		
		return $output;
	}

	function DirectIOSave($file = '', $path)
	{
		$c_id = $this->f_no; // on sauve le numero actuel
		$this->Open($file);
		$this->SaveTemplate($path);
		$this->f_no = $c_id; // on remet le numero
	}

	function DefineTemplate($arr)
	{
		if(!is_array($arr))
		{
			$this->_Error(12);
			return;
		}

		foreach($arr as $key => $val)
		{
			$this->Open($val);
			$this->def_tpl[count($this->f)-1] = $key;
		}
	}

	function SetTemplate($key)
	{
		$this->ChangeTemplate($key);
	}
	
	function SetTemplateID($key)
	{
		$this->ChangeTemplate($key);
	}

	function ChangeTemplate($key)
	{
		if(is_string($key))
		{ 
			// on prend l'id
			if(!in_array($key, $this->def_tpl, true))
			{
				$this->_Error(11, $key);
				return;
			}

			$key = array_search($key, $this->def_tpl, true);
		}

		if(is_int($key))
		{
			if($key < 0 || $key >= count($this->f))
			{
				$this->_Error(10, $key);
			}
		}

		$this->f_no = $key;
	}

	function ParseGlobals()
	{
		$this->f[$this->f_no]['php_items'] = $this->_CaptureItems('', 'PHP');

		if(count($this->f[$this->f_no]['php_items']) == 0)
		{
			return;
		}

		@extract($GLOBALS, EXTR_SKIP || EXTR_REFS);
		foreach($this->f[$this->f_no]['php_items'] as $item)
		{
			$replace = '$'.$item;
			if(!eregi("\(", $replace)) // protection securit des methodes et fonctions !
				{
					@eval("\$tmp = $replace;");

				$item = '$'.$item;
				$item = str_replace('$', '\$', $item);
				$item = str_replace('[', '\[', $item);
				$item = str_replace(']', '\]', $item);
				$this->f[$this->f_no]['buffer'] = $this->_ReplaceItem($item, $tmp, $this->f[$this->f_no]['buffer']);
			}
		}
	}

	function Parse($path, $replace, $functions = '')
	{ 
		// fonction de formatage des donnes
		$replace = $this->_ApplySpecialFunction($replace, $functions);

		if($this->_IsBlocDesired($path)) // est ce un bloc ?
		{
			$item = $this->_GetItem($path); // on prend l'item
			$fathers_arr = $this->_GetFathers($path, 'ARRAY'); // les pres ds un array
			$bloc = $this->_GetFather($path); // on prend le pre
			
			if(!$this->BlocExists($bloc))
				$this->_Error(2, $this->f[$this->f_no]['name'], $bloc);

			$this->_ItemVerify($item, $bloc); // verification
			
			if(!$this->_IsDefined($path)) // le chemin est il enregistr
				$this->_DefineBloc($path); // defini les blocs un par un

			$this->_DirectParseInBloc($fathers_arr, $item, $replace);
		}
		else
		{
			$this->_ItemVerify($path); // verification
			$this->_DirectParseInFile($path, $replace); // parsing direct dans le fichier
		}
	}

	function _DirectParseInFile($item, $replace)
	{
		$this->f[$this->f_no]['buffer'] = $this->_ReplaceItem($item, $replace, $this->f[$this->f_no]['buffer']);
	}

	function _DirectParseInBloc($fathers_arr, $item, $replace)
	{
		$b_ref = & $this->f[$this->f_no]['def_blocs']; 
		// on atteint le bloc
		for($i = 0;$i < count($fathers_arr);$i++)
		{
			$b_ref = & $b_ref[$fathers_arr[$i]];

			if($i < count($fathers_arr)-1)
				$b_ref = & $b_ref['children'];
		}
		
		// on regarde si il y eu un loop > 0
		$loop_nb = count($b_ref['parsed']);
		$b_ref['parsed'][$loop_nb-1] = $this->_ReplaceItem($item, $replace, $b_ref['parsed'][$loop_nb-1]);
	}

	function FastParse($path, $functions = '')
	{ 
		// Est ce un bloc alors on remplace
		if($this->_IsBlocDesired($path))
			$item = $this->_GetItem($path); // on prend le dernier item
		else
			$item = $path;
		$this->Parse($path, $GLOBALS[$item], $functions);
	}

	function ParseDBRow($bloc, $msg = '', $func = '')
	{
		$this->_PathVerify($bloc);

		if(empty($this->NavColorFirst))$this->NavColorFirst = TPLN_DB_NavColorFirst;
		if(empty($this->NavColorSecond))$this->NavColorSecond = TPLN_DB_NavColorSecond;		
		
		$res = $this->GetRowsCount();
		if($res == 0)
		{
			$this->parseBloc($bloc, $msg);
		}
		else
		{			
			$i = 0;
			while($row = $this->DBFetch())
			{
				$keys = @array_keys($row); // prise du mon des clefs
				$i++;
	
				foreach($keys as $key)
				{
					if($this->ItemExists($key, $bloc))
					{
						//if(!empty($func))
							$this->Parse("$bloc.$key", $row[$key], $func);
						//else
							//$this->Parse("$bloc.$key", $row[$key]);
					}
					
					if($this->ItemExists('_NavColor', $bloc))
					{
						$color = ($i % 2) ? $this->NavColorFirst : $this->NavColorSecond;
						$this->Parse($bloc.'._NavColor', $color);
				   }
					
				}
				$this->Loop($bloc);
			}
		}
	}

	function ParseDBField($bloc, $func = '')
	{
		$fields = $this->GetFields();
	
		// verification du chemin
		$this->_PathVerify($bloc);
		
		// on simule que l'on prend un item
		// $last_bloc = $this->_GetItem($bloc); 

		if(empty($this->NavColorFirst))$this->NavColorFirst = TPLN_DB_NavColorFirst;
		if(empty($this->NavColorSecond))$this->NavColorSecond = TPLN_DB_NavColorSecond;				
		
		// control  faire sur le chemin !
		for($i = 0; $i < count($fields); $i++)
		{
			$field_name = $fields[$i];

			// if($this->ItemExists($field_name, $last_bloc))
			if($this->ItemExists('_Field', $bloc))
			{
				//if(!empty($func))
					$this->Parse("$bloc._Field", $field_name, $func);
				//else
					//$this->Parse("$bloc._Field", $field_name);

				if($this->ItemExists('_NavColor', $bloc))
				{
					$color = ($i % 2) ? $this->NavColorFirst : $this->NavColorSecond;
					$this->Parse($bloc.'._NavColor', $color);
				}
					
			}
			$this->Loop($bloc);
		}
	}

	function LoadArrayInBloc($path, $arr, $msg = '', $looped = false)
	{			
		if(count($arr) == 0)
		{
			if(!$looped)
			{
				$this->ParseBloc($path, $msg);
			}
			else
			{
				$this->EraseBloc($path, $msg);
				$this->Loop($path);
			}
		}
		else
		{
			foreach($arr as $current_arr)
			{
				$sub_blocs = array();
				$keys = @array_keys($current_arr);
				foreach($keys as $key)
				{
					if(!is_array($current_arr[$key]))
					{
						$zpath = $path;
						if($looped)
						{
							$ttmp = explode('.', $zpath);
							$zpath = $ttmp[count($ttmp)-1];
						}
						
						if($this->ItemExists($key, $zpath))
						{
							//echo "parse: $path.$key".'<br />';
							$this->Parse("$path.$key", $current_arr[$key]);						
						}
					}
					else
					{
						$sub_bloc = $key;
						$this->LoadArrayInBloc($path.'.'.$sub_bloc, $current_arr[$key], $msg, true);					
					}
				}
				//echo "loop: $path".'<hr />';
				$this->Loop($path);			
			}
		}
	}

	function Loop($path)
	{ 
		// vrification de l'existence du chemin comme defini
		if(!$this->_IsDefined($path, 'NOITEM'))
		{
			$this->_Error(4.1, $this->f[$this->f_no]['name'], $path);
			return;
		}

		$fathers_arr = $this->_GetFathers($path, 'ARRAY', 0); 
		
		// touche le bloc
		$b_ref = & $this->f[$this->f_no]['def_blocs'];

		for($i = 0;$i < count($fathers_arr);$i++)
		{
			$b_ref = & $b_ref[$fathers_arr[$i]];

			if($i < count($fathers_arr)-1)
			{
				$b_ref = & $b_ref['children'];
			}
		}
		
		// toute la gnration qui suit est remplac donc
		if(count($b_ref['children']) > 0)
		{ 
			// $level = count($fathers_arr)-1;
			$child_blocs = $this->_GetNextGenerationBlocs($fathers_arr); 
			// on encapsule les blocs
			$this->_EncapsuleBlocs($fathers_arr, $child_blocs);
		}
		
		// on ajoute une session parse
		// on incrmente la valeur de loop
		$b_ref['parsed'][] = $b_ref['structure'];
		$b_ref['is_looped'] = 1;
	}

	function _GetNextGenerationBlocs($bloc_arr)
	{ 
		// touche le bloc
		$b_ref = & $this->f[$this->f_no]['def_blocs'];

		for($i = 0;$i < count($bloc_arr);$i++)
		{
			$b_ref = & $b_ref[$bloc_arr[$i]];

			if($i < count($bloc_arr)-1)
			{
				$b_ref = & $b_ref['children'];
			}
		}

		$b_names = array_keys($b_ref['children']);

		return $b_names;
	}

	function _EncapsuleBlocs($bloc_arr, $child_blocs)
	{ 
		// on encapsule chacun des enfants CAD
		// rtrcissement du bloc s'il a des enfants
		// parsing chez le pre du fils parsed
		// remize  zero du bloc fils
		foreach($child_blocs as $children)
		{ 
			// touche le bloc
			$b_ref = & $this->f[$this->f_no]['def_blocs'];

			for($i = 0;$i < count($bloc_arr);$i++)
			{
				$b_ref = & $b_ref[$bloc_arr[$i]];

				if($i == count($bloc_arr)-1)
					$father_parsed = & $b_ref['parsed'][count($b_ref['parsed'])-1]; // c'est le dernier

				$b_ref = & $b_ref['children'];
			}

			$children_all_parsed = & $b_ref[$children]['parsed'];
			$children_structure = & $b_ref[$children]['structure'];
			$children_parsed = ''; 
			// bloc
			if(count($children_all_parsed) == 1)
				$children_parsed = $children_all_parsed['0'];

			if(count($children_all_parsed) > 1)
			{
				for($l = 0; $l < count($children_all_parsed)-1; $l++)
					$children_parsed = $children_parsed.$children_all_parsed[$l];
			}
			// parsing chez le pre
			$father_parsed = $this->_ReplaceBloc($children, $children_parsed, $father_parsed); 
			// remize  zero du bloc child
			$children_all_parsed = array($children_structure);
			unset($children_parsed);
		}
	}

	function EraseItem($item)
	{
		if(is_array($item))
		{
			foreach($item as $tmp)
				$this->Parse($tmp, '');
		}
		else
			$this->Parse($item, '');
			
	}

	function EraseBloc($path, $msg='')
	{
		if(is_array($path))
		{
			foreach($path as $tmp)
				$this->ParseBloc($tmp, $msg);
		}
		else
			$this->ParseBloc($path, $msg);
	}

	function GetFile($filename, $blocname = '')
	{
		if(!$fp = @fopen($filename, 'r')) // ouverture du fichier en lecture
		{
			$this->_Error(0, $filename);
		}

		$filebuffer = @fread($fp, filesize($filename));
		@fclose($fp);
		clearstatcache(); // efface la mmoire tampon
		
		if(!empty($blocname)) // on veut capturee un bloc
			$filebuffer = $this->_CaptureBloc($blocname, $filebuffer);

		return $filebuffer;
	}

	function ParseBloc($path, $replace)
	{
		$this->_PathVerify($path); // Verification de l'existence des blocs du chemin
		 
		// definition du chemin
		if(!$this->_Isdefined($path, 'NOITEM')) $this->_DefineBloc($path.'.none'); 
		
		// bloc imbriqu ?
		if(!$this->_IsBlocDesired($path))
		{
			$this->f[$this->f_no]['buffer'] = $this->_ReplaceBloc($path, $replace, $this->f[$this->f_no]['buffer']);
			$bloc_arr[] = $path;
		}
		else
		{ 
			// on doit aller dans le bloc pour l'liminer !
			$bloc_arr = $this->_GetFathers($path, 'ARRAY', 0);
		}
		
		// on efface le bloc
		$b_ref = & $this->f[$this->f_no]['def_blocs'];
		
		// un seul bloc ?
		if(count($bloc_arr) == 1)
		{
			$b_ref = & $b_ref[$bloc_arr[0]];
		}
		else
		{ 
			// on efface dans le dernier pere le bloc en cours
			for($i = 0;$i < count($bloc_arr);$i++)
			{
				$b_ref = & $b_ref[$bloc_arr[$i]];

				if($i == count($bloc_arr)-2)
					$last_father_parsed = & $b_ref['parsed'][count($b_ref['parsed'])-1];

				if($i < count($bloc_arr)-1)
					$b_ref = & $b_ref['children'];
			}

			$last_father_parsed = $this->_ReplaceBloc($bloc_arr[count($bloc_arr)-1], $replace, $last_father_parsed);
		}

		$b_ref['parsed'] = array(); // on efface sa session du dernier bloc !
	}

	function ReloadBlocVars($path)
	{
		$this->_PathVerify($path);

		$last_bloc = $this->_GetItem($path);
		$this->f[$this->f_no]['shortcut_blocs'][$last_bloc]['items'] = $this->_CaptureItems($last_bloc);
	}

	function IncludeFile($item, $file)
	{ 
		// verification pour un bloc
		$this->Parse($item, "{#include(\"$file\");}"); // on replace par la commande include
		$this->f[$this->f_no]['cmd_items'][] = $file; // on ajoute  la liste des f_cmd
		
	}

	function GetBlocInFile($path)
	{
		$this->_PathVerify($path);
		$all_bloc = explode('.', $path);
		$bloc_name = $all_bloc[count($all_bloc)-1];
		$bloc = $this->_CaptureBloc($bloc_name, $this->f[$this->f_no]['buffer']);
		
		return $bloc;
	}

	function Write()
	{
		$this->_Out();
		echo $this->f[$this->f_no]['buffer'];
		$this->_CreateCache(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['name']);
		if(TPLN_AUTO_UNSET_TEMPLATE) $this->_initTemplateVars();
	}

	function Output()
	{
		$this->_Out();
		$this->_CreateCache(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['name']);
		$buffer = $this->f[$this->f_no]['buffer'];
		if(TPLN_AUTO_UNSET_TEMPLATE) $this->_initTemplateVars();

		return $buffer;
	}

	function SaveTemplate($path)
	{ 
		// ajout d'un fonction pour la cration de dossier
		$act_dir = '';
		$all_dir = explode('/', $path);

		if(count($all_dir) > 1)
		{
			for($i = 0;$i < count($all_dir)-1;$i++)
			{
				if($i > 0)$act_dir .= '/';
				$act_dir .= $all_dir[$i];

				if(!is_dir($act_dir))
				{
					if(!@mkdir($act_dir, 0755))
					{
						$this->_Error(7.1, $act_dir);
						return;
					}

					clearstatcache();
				}
			}
		}

		$output = $this->Output();
		$fp = @fopen($path, 'w'); // ouverture du fichier
		@fwrite($fp, $output);
		@fclose($fp);
		clearstatcache(); // efface la mmoire tampon
	}

	// Additionnel ***********************************************************************************************************************
	function _ApplySpecialFunction($replace = '', $functions = '')
	{
		if(empty($functions))return $replace;

		// y a t'il une fonction spciale ?
		$functions = explode('|', $functions);

		foreach($functions as $function_name)
		{
			if($function_name != 'B' && $function_name != 'I' && $function_name != 'U' && $function_name != 'S' && !empty($function_name))
			{
				$replace = $function_name($replace);
			}
		}
		// Bold, Italic, Underline
		if(in_array('B', $functions))$replace = "<strong>$replace</strong>";
		if(in_array('I', $functions))$replace = "<i>$replace</i>";
		if(in_array('U', $functions))$replace = "<u>$replace</u>";
		if(in_array('S', $functions))$replace = "<s>$replace</s>";

		return $replace;
	}


	// remplace le fils dans le pre
	function _ParseEncapsuledBlocs($bloc)
	{ 
		// on atteint le dernier bloc
		// on conserve son pre (adresse) et son fils (nom, structure)
		// on supprime le dernier membre de bloc
		while(count($bloc) > 1)
		{
			$b_ref = & $this->f[$this->f_no]['def_blocs'];

			for($i = 0;$i < count($bloc);$i++)
			{
				$b_ref = & $b_ref[$bloc[$i]];

				if($i == count($bloc) - 2) // capture du dernier pre
					$father_parsed = & $b_ref['parsed'][count($b_ref['parsed'])-1];

				if($i == count($bloc) - 1) // capture du dernier fils
				{
					$children_name = $bloc[$i];
					$children_all_parsed = $b_ref['parsed'];
					$children_parsed = null;

					if(count($children_all_parsed) > 1)
					{
						for($j = 0;$j < count($children_all_parsed)-1;$j++)
							$children_parsed .= $children_all_parsed[$j];
					}
					else
					{
						if(count($children_all_parsed) == 1)
							$children_parsed = $children_all_parsed[0];
					}
				}

				$b_ref = & $b_ref['children'];
			}

			// on remplace le fils dans le pre
			$father_parsed = $this->_ReplaceBloc($children_name, $children_parsed, $father_parsed);
			
			// on supprime le dernier bloc
			unset($children_parsed);
			$bloc = array_slice($bloc, 0, count($bloc)-1);
		}
	}

	function _ParseSubBlocs($level, $bloc_arr)
	{
		$b_ref = & $this->f[$this->f_no]['def_blocs'];

		foreach($bloc_arr as $cur_bloc)
		{
			$b_ref = & $b_ref[$cur_bloc]['children'];
		}

		if(!is_array($b_ref))return;

		$b_names = array_keys($b_ref); // contient tous les noms de blocs
		
		foreach($b_names as $bloc)
		{
			if(count($b_ref[$bloc]['children']) > 0)
			{
				$bloc_arr[] = $bloc;
				$this->_ParseSubBlocs($level + 1, $bloc_arr);
			}
			else
			{ 
				$bloc_arr = array_slice($bloc_arr, 0, $level + 1);
				$bloc_arr[] = $bloc;
				$this->_ParseEncapsuledBlocs($bloc_arr);
			}
		}
	}

	function _ParseBigFathers()
	{ 
		// on prend les pres parsed que l'on remplace dans le buffer
		$b_ref = & $this->f[$this->f_no]['def_blocs']; 
		// on prend les noms du parent BIG
		$b_names = array_keys($b_ref);
		$b_parsed = ''; // initialisation
		
		foreach($b_names as $bloc)
		{
			if(count($b_ref[$bloc]['parsed']) == 1 && array_key_exists('0', $b_ref[$bloc]['parsed']))
				$b_parsed = $b_ref[$bloc]['parsed'][0];
				
			// on regarde si le bloc a t en loop
			if(count($b_ref[$bloc]['parsed']) > 1)
			{ 
				// on ne prend pas la premire
				for($i = 0;$i < count($b_ref[$bloc]['parsed'])-1;$i++)
					$b_parsed .= $b_ref[$bloc]['parsed'][$i];
			}

			$this->f[$this->f_no]['buffer'] = $this->_ReplaceBloc($bloc, $b_parsed, $this->f[$this->f_no]['buffer']); 
			// unset($b_parsed);
			$b_parsed = '';
		}
	}

	function _GetBigBlocs()
	{ 
		// touche le bloc
		$b_ref = & $this->f[$this->f_no]['def_blocs'];
		$b_names = array_keys($b_ref);
		return $b_names;
	}

	function _Out()
	{
		if(count($this->f[$this->f_no]['def_blocs']) > 0)
		{
			$b_ref = & $this->f[$this->f_no]['def_blocs']; 
			// on obtiens les noms des big blocs
			$big_blocs = $this->_GetBigBlocs();

			foreach($big_blocs as $bloc)
			{
				if(count($b_ref[$bloc]['children']) > 0)
					$this->_ParseSubBlocs(0, (array)$bloc);
			}

			$this->_ParseBigFathers(); // va compresser les bloc big et les parse au sein du buffer
		}

		$this->_ParseAllIncludeCmd();
		$this->_ParseChrono();
		$this->_ParseLogo();
		$this->_ParseVersion();
		$this->_ParseQueryCount();

		if(TPLN_AUTO_CLEAN_BLOCS) $this->CleanBLocs(); 
		// compression ?
		if($this->HTMLCompress)
		{
			$this->f[$this->f_no]['buffer'] = preg_replace("/(\r\n|\n)/", '', $this->f[$this->f_no]['buffer']);
		}
	}


	function _CaptureItems($subject = '', $type = '')
	{
		if(empty($subject))
		{
			$subject = $this->f[$this->f_no]['buffer']; // ds le fichier
			$blocs = $this->f[$this->f_no]['shortcut_blocs']['all'];
		}
		else
		{
			$subject = $this->_CaptureBloc($subject, $this->f[$this->f_no]['buffer']); 
			// on capture les blocs du sujet
			$blocs = $this->_CaptureAllBlocs($subject); 
			// on les efface
			foreach($blocs as $bloc)
				$subject = $this->_ReplaceBloc($bloc, '', $subject);
		}
		// exclusion des espaces etc
		if($type == 'PHP')
		{
			$motif = "/\{\\$([^ ;\*\$\\\,\\n\\t]+)?\}/msU";
		}elseif($type == 'CONSTANT')
		{
			$motif = "/\{CONST::([^ ;\.\*\$\\\,\\n\\t]+)?\}/msU";
		}
		else
		{
			$motif = "/\{([^ ;\.\*\$\\\,\\n\\t]+)?\}/msU";
		}

		$match = preg_match_all($motif, $subject, $tab);
		
		// on ddoublonne le tableau
		$one = array();

		if(count($tab[1]) > 0)
			$one = array_unique($tab[1]);

		return $one;
	}
	
	// fonction qui capture un bloc sans ses motif
	function _CaptureBloc($name, $subject)
	{
		if(empty($name))
		{
			$this->_Error(5, $this->f[$this->f_no]['name']);
			return;
		}

		$motif = "<bloc::$name>(.*)?<\\/bloc::$name>";
		$match = @preg_match("/$motif/msU", $subject, $bloc); 
		// error
		if(!$match)
		{
			$this->_Error(2, $this->f[$this->f_no]['name'], $name);
			return;
		}

		$bloc = $bloc[1];
		$bloc = @rtrim($bloc);

		return $bloc;
	}
	// fonction qui capture tous les blocs du sujet
	function _CaptureAllBlocs($subject = '')
	{
		if(empty($subject))
		{
			$subject = & $this->f[$this->f_no]['buffer'];
		}

		$motif = "<bloc::([^ ;\.\*\$\\\,\\n\\t]+)?>";
		$match = preg_match_all("/$motif/U", $subject, $blocs);

		return $blocs[1];
	}
	// capture les items de tous les blocs du fichiers
	function _CaptureItemsInEachBloc()
	{
		if(count($this->f[$this->f_no]['shortcut_blocs']['all']) == 0)
			return;

		foreach($this->f[$this->f_no]['shortcut_blocs']['all'] as $bloc)
			$this->f[$this->f_no]['shortcut_blocs'][$bloc]['items'] = $this->_CaptureItems($bloc);
	}
	// preds les blocs du sujet
	function _GetAllBlocs($subject)
	{
		$motif = "<bloc::([^ ;\.\*\$\\\,\\n\\t]+)?>";
		preg_match_all("/$motif/U", $subject, $blocs);
		return $blocs[1];
	}
	// capture les instructions include au sein d'un template
	function _CaptureIncludeCmd()
	{
		$motif = "\{#include\(([^ ;\*,\\n\\t]+)?\);\}";
		$match = preg_match_all("/$motif/U", $this->f[$this->f_no]['buffer'], $tab);

		if(count($tab[1]) == 0)
			return;

		$one = array_unique($tab[1]);
		$this->f[$this->f_no]['cmd_items'] = $one;
	}

	
	function ItemExists($item_name, $bloc = '')
	{
		if(empty($bloc))
		{
			if(@in_array($item_name, $this->f[$this->f_no]['items']))
				return true;
			else
				return false;
		}
		else
		{
			if(@in_array($item_name, $this->f[$this->f_no]['shortcut_blocs'][$bloc]['items']))
				return true;
			else
				return false;
		}
	}

	function _ItemVerify($item, $bloc = '')
	{
		if(empty($bloc))
		{
			if(!$this->ItemExists($item))
			{
				$this->_Error(1, $this->f[$this->f_no]['name'], '', $item);
				return;
			}
		}
		else
		{
			if(!$this->ItemExists($item, $bloc))
			{
				$this->_Error(1.1, $this->f[$this->f_no]['name'], $bloc, $item);
				return;
			}
		}
	}
	
	// verify que tous les blocs d'un chemin existe
	function _PathVerify($path)
	{
		$tab = @explode('.', $path); 
		// on atteint le bloc et on lui ajoute les items
		for($i = 0; $i < count($tab); $i++)
		{
			if(!$this->BlocExists($tab[$i]))
			{
				$this->_Error(2, $this->f[$this->f_no]['name'], $tab[$i]);
				return;
			}
		}
	}

	function BlocExists($bloc_name)
	{
		if(@in_array($bloc_name, $this->f[$this->f_no]['shortcut_blocs']['all']))
			return true;
		else
			return false;
	}

	// verifie l'existence de la fin du bloc
	function _EndBlocVerify()
	{
		foreach($this->f[$this->f_no]['shortcut_blocs']['all'] as $bloc_name)
		{
			$motif = "/<\\/bloc::$bloc_name>/U";
			if(!preg_match($motif, $this->f[$this->f_no]['buffer']))
			{
				$this->_Error(8, $this->f[$this->f_no]['name'], $bloc_name);
				return;
			}
		}
	}
	// verifie qu'il n'existe pas deux bloc similaires (bloc en double)
	function _DualBlocVerify()
	{
		if(count($this->f[$this->f_no]['shortcut_blocs']['all']) == 0)
			return;

		$blocs = array();
		foreach($this->f[$this->f_no]['shortcut_blocs']['all'] as $bloc)
		{
			if(!in_array($bloc, $blocs))
				$blocs[] = $bloc;
			elseif(!in_array($bloc, $this->blocs_double_exceptions))
				$this->_Error(2.1, $this->f[$this->f_no]['name'], $bloc);
		}
	}
	// ajout des exceptions de blocs
	function addBlocException($str)
	{
		if(is_array($str))
		{
			foreach($str as $bloc)
			$this->addBlocException($bloc);
		}
		else
		{
			if(!in_array($str, $this->blocs_double_exceptions))
				$this->blocs_double_exceptions[] = $str;
		}
	}

	// item replace
	function _ReplaceItem($name, $replace, $subject)
	{
		if(empty($name))
		{
			$this->_Error(6, $this->f[$this->f_no]['name']);
			return;
		}

		$str = preg_replace("/\{$name\}/U", $replace, $subject);
		return $str;
	}

	// fonction qui replace un bloc sans ses motifs
	function _ReplaceBloc($name, $replace, $subject)
	{
		if(empty($name))
		{
			$this->_Error(5, $this->f[$this->f_no]['name']);
			return;
		}
		// c'est pas un tableau
		if(!is_array($name) && !is_array($replace))
		{
			$motif = "<bloc::$name>(.*)?<\\/bloc::$name>";
			$str = @preg_replace("/$motif/msU", $replace, $subject);
		}
		else
		{
			$str = @preg_replace($name, $replace, $subject);
		}
		return $str;
	}

	function _ParseIncludeCmd($file)
	{
		// parsing du nom de fichier
		$file = str_replace(chr(34), ' ', $file);
		$file = str_replace(chr(39), ' ', $file);
		$file = trim($file);

		$filebuffer = $this->GetFile($file);

		if($this->_IsPhpFile($file))
		{
			$filebuffer = $this->_EvalHtml($filebuffer);
		}

		$file = str_replace('/', '\/', $file);
		$motif = "/\{\#include\(\"$file\"\)\;\}|\{\#include\(\'$file\'\)\;\}/";
		$this->f[$this->f_no]['buffer'] = @preg_replace($motif, $filebuffer, $this->f[$this->f_no]['buffer']);
	}

	function _ParsePhpCommands()
	{
        // on parse les if
        $motif = "#{\#(.*|^include)}#";
		preg_match_all($motif, $this->f[$this->f_no]['buffer'], $matches);

        if(count($matches) == 2 && $matches[1] > 0)
        {
            // on remplace par des variables globales
            foreach($matches[1] as $m)
			{			
                $m2 = $m;
				preg_match_all('#\$([[:alnum:]|\_]*)#', $m, $ms);
				
                if(count($ms) == 2)
				{					
                   foreach($ms[0] as $tm)
                   {				   		
                    	if(!in_array($tm, array('$_GET', '$_POST', '$_COOKIE', '$_SERVER', '$_SESSION', '$_REQUEST')))
                        	$m2 = str_replace($tm, '$GLOBALS["'.str_replace('$','',$tm).'"]', $m);						
                   }
                }
				$this->f[$this->f_no]['buffer'] = str_replace("{#$m}", "{#$m2}", $this->f[$this->f_no]['buffer']);
            }

			$this->f[$this->f_no]['buffer'] = str_replace("{#else}", "{#else:}", $this->f[$this->f_no]['buffer']);
            $this->f[$this->f_no]['buffer'] = str_replace('{#endif}', '{#endif;}', $this->f[$this->f_no]['buffer']);
			$this->f[$this->f_no]['buffer'] = preg_replace("#{\#(.*)\)}#", "{#$1):}", $this->f[$this->f_no]['buffer']);
			$this->f[$this->f_no]['buffer'] = preg_replace($motif, "<?php $1 ?>" ,$this->f[$this->f_no]['buffer']);
			$this->f[$this->f_no]['buffer'] = $this->_EvalHtml($this->f[$this->f_no]['buffer']);
		}
    }



	function _ParseAllIncludeCmd()
	{
		if(count($this->f[$this->f_no]['cmd_items']) == 0)
			return;

		foreach($this->f[$this->f_no]['cmd_items'] as $filename)
			$this->_ParseIncludeCmd($filename);
	}

	function _EvalHtml($string)
	{
		// $string = implode('', $string);
		ob_start();
		eval('?>'.$string);
		$string = ob_get_contents();
		ob_end_clean();
		return $string;
	}
	// est ce un fichier php ?
	function _IsPhpFile($filename)
	{
		$php_file_extension = 'php|phtml|php4|php3'; // extensions de fichiers php

		if(preg_match("/\.($php_file_extension)$/i", basename($filename)))
			return true;

		return false;
	}


	function _IsBlocDesired($path)
	{
		if(preg_match("/[.]/", $path))
			return true;
		else
			return false;
	}
	// on vrifie que dans tous les blocs sont dfinis
	function _IsDefined($path, $type = '')
	{
		if(empty($type))
			$fathers_path = $this->_GetFathers($path); // rcupre les pres

		if($type == 'NOITEM')
			$fathers_path = $path; // rcupre les pres

		if(@in_array($fathers_path, $this->f[$this->f_no]['shortcut_blocs']['used'], true))
			return true;
		else
			return false;
	}

	// recupere le texte du bloc
	// recupre l'item d'un path
	function _GetItem($path)
	{ 
		// if(!preg_match("/[.]/",$path)) {return;}
		$path_arr = explode('.', $path);
		return $path_arr[count($path_arr)-1];
	}
	
	// recupre le pre donc avant dernier bloc
	function _GetFather($path)
	{ 
		// if(!preg_match("/[.]/",$path)) {return;}
		$path_arr = explode('.', $path);
		return $path_arr[count($path_arr)-2];
	}
	
	// recupere les pres
	function _GetFathers($path, $type = '', $with_item = 1)
	{ 
		// if(!preg_match("/[.]/",$path)) {return;}
		$path_arr = explode('.', $path);

		if($with_item == 1)
			$fathers_arr = array_slice($path_arr, 0, count($path_arr)-1); // tous les noms sauf le dernier
		else
			$fathers_arr = $path_arr;

		if($type == 'ARRAY')
			return $fathers_arr;
		else
		{
			$fathers_path = join('.', $fathers_arr);
			return $fathers_path;
		}
	}

	function _DefineBloc($path)
	{
		$fathers_path = $this->_GetFathers($path); // pas d'item
		$fathers_arr = $this->_GetFathers($path, 'ARRAY'); // array
		
		foreach($fathers_arr as $bloc)
		{
			$cur_arr[] = $bloc;

			if(count($cur_arr) == 1)
				$cur_path = $cur_arr[0];
			else
				$cur_path = join('.', $cur_arr);
				
			// definition de tous les pres
			if(!$this->_IsDefined($cur_path, 'NOITEM'))
			{
				$this->_SavePath($cur_path);
				$a_bloc = $this->_InitialiazeBloc($bloc); 
				// stockage
				$b_ref = & $this->f[$this->f_no]['def_blocs'];

				$i = 0;
				foreach($cur_arr as $cur_bloc)
				{
					$b_ref = & $b_ref[$cur_bloc];
					if($i < (count($cur_arr)-1))
						$b_ref = & $b_ref['children'];
					$i++;
				}

				$b_ref = $a_bloc;
			}
		}
	}

	function _InitialiazeBloc($bloc)
	{
		$cur['structure'] = $this->_CaptureBloc($bloc, $this->f[$this->f_no]['buffer']); // structure
		$cur['parsed'][] = $cur['structure']; // parsed
		$cur['is_looped'] = 0;
		$cur['children'] = array();

		return $cur;
	}

	function _SavePath($fathers_path)
	{
		$this->f[$this->f_no]['shortcut_blocs']['used'][] = $fathers_path; // on l'enregistre dans les blocs dfinis
	}


	// code from bitlux
	function XSSProtect($string)
	{
		if(is_array($string))
		{
			foreach($string as $key => $val)
			$string[$key] = $this->XSSProtect($val);

			return $string;
		}

		if (get_magic_quotes_gpc())$string = stripslashes($string);

		$string = str_replace(array("&amp;", "&lt;", "&gt;"), array("&amp;amp;", "&amp;lt;", "&amp;gt;",), $string); 
		// fix &entitiy\n;
		$string = preg_replace('#(&\#*\w+)[\s\r\n]+;#U', "$1;", $string);
		$string = @html_entity_decode($string, ENT_COMPAT, "UTF-8"); 
		// remove any attribute starting with "on" or xmlns
		$string = preg_replace('#(<[^>]+[\s\r\n\"\'])(on|xmlns)[^>]*>#iU', "$1>", $string); 
		// remove javascript: and vbscript: protocol
		$string = preg_replace('#([a-z]*)[\s\r\n]*=[\s\n\r]*([\`\'\"]*)[\\s\n\r]*j[\s\n\r]*a[\s\n\r]*v[\s\n\r]*a[\s\n\r]*s[\s\n\r]*c[\s\n\r]*r[\s\n\r]*i[\s\n\r]*p[\s\n\r]*t[\s\n\r]*:#iU', '$1=$2nojavascript...', $string);
		$string = preg_replace('#([a-z]*)[\s\r\n]*=([\'\"]*)[\s\n\r]*v[\s\n\r]*b[\s\n\r]*s[\s\n\r]*c[\s\n\r]*r[\s\n\r]*i[\s\n\r]*p[\s\n\r]*t[\s\n\r]*:#iU', '$1=$2novbscript...', $string); 
		// <span style="width: expression(alert('Ping!'));"></span>
		// only works in ie...
		$string = preg_replace('#(<[^>]+)style[\s\r\n]*=[\s\r\n]*([\`\'\"]*).*expression[\s\r\n]*\([^>]*>#iU', "$1>", $string);
		$string = preg_replace('#(<[^>]+)style[\s\r\n]*=[\s\r\n]*([\`\'\"]*).*s[\s\n\r]*c[\s\n\r]*r[\s\n\r]*i[\s\n\r]*p[\s\n\r]*t[\s\n\r]*:*[^>]*>#iU', "$1>", $string); 
		// remove namespaced elements (we do not need them...)
		$string = preg_replace('#</*\w+:\w[^>]*>#i', "", $string); 
		// remove really unwanted tags
		do
		{
			$oldstring = $string;
			$string = preg_replace('#</*(style|script|embed|object|iframe|frame|frameset|ilayer|layer|bgsound|title|base)[^>]*>#i', "", $string);
		}
		while ($oldstring != $string);

		return $string;
	}

	function _ParseVersion()
	{
		if($this->ItemExists('_Version')) // on place le logo
			$this->Parse('_Version', TPLN_VERSION);
	}

	function _ParseQueryCount()
	{
		if($this->ItemExists('_QueryCount')) // on place le logo
			$this->Parse('_QueryCount', $this->query_count);
	}


	function _ParseLogo()
	{
		if($this->ItemExists('_Logo')) // on place le logo
			{
				$this->Parse('_Logo', '<a href="http://tpln.sourceforge.net"><img src="http://tpln.sourceforge.net/logo.gif" alt="made with TPLN Template!" border="0" target="_blank"  /></a>');
		}
	}

	function SetChrono($type = 'this')
	{
		$this->chrono_type[$this->f_no] = $type;
	}

	function _ParseChrono()
	{
		if($this->ItemExists('_Chrono')) // on place le chrono
		{
			$this->_GetExecutionTime();
			$this->Parse('_Chrono', $this->f[$this->f_no]['execution_time']);
		}
	}
	
	// gestion du chrono
	function _GetExecutionTime()
	{
		$time = explode(' ', microtime());
		$fin = $time[1] + $time[0]; 
		// on veut les perf de cette session ou tous
		if($this->chrono_type[$this->f_no] == 'ALL')
			$this->f[$this->f_no]['execution_time'] = intval(10000 * ((double)$fin - (double)TPLN_CHRONO_STARTED)) / 10000;
		else
			$this->f[$this->f_no]['execution_time'] = intval(10000 * ((double)$fin - (double)$this->f[$this->f_no]['chrono_started'])) / 10000;
	}


	// retourne la valeur de la premire ligne
	function _InCachePeriod()
	{
		$expire = $this->_GetTime(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['cache_name']);

		if($expire >= $this->f[$this->f_no]['time_started'])
		{
			return true;
		}
		else
		{
			@unlink(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['cache_name']); // on efface le fichier
			return false;
		}
	}
	// prend le temps au dessus du fichier
	function _GetTime()
	{
		$fp = @fopen(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['cache_name'], 'r'); // ouvre le fichier
		$expire = trim(@fgets($fp, 12)); 
		// $data = fread($fp, filesize($file));
		@fclose($fp);

		clearstatcache();

		return $expire;
	}

	function _CreateCache()
	{
		if(!$this->f[$this->f_no]['create_cached_file'])
		{
			return;
		}
		else
		{ 
			// ajout d'un fonction pour la cration de dossier
			$all_dir = explode('/', $this->f[$this->f_no]['cache_name']);
			if(count($all_dir) > 0)
			{
				$count = count($all_dir)-1; // on veut le dernier
				$act_dir = TPLN_CACHE_DIR;
				for($i = 0;$i < $count;$i++)
				{
					$act_dir .= '/'.$all_dir[$i];
					if(!is_dir($act_dir))
					{
						if(!@mkdir($act_dir, 0755))
						{
							$this->_Error(7);
							return;
						}

						clearstatcache();
					}
				}
			}

			$cache_file_content = $this->f[$this->f_no]['cache_expire']."\r".$this->f[$this->f_no]['buffer'];
			$fp = @fopen(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['cache_name'], 'w'); // ouverture du fichier
			@fwrite($fp, $cache_file_content);
			@fclose($fp);
			clearstatcache(); // efface la mmoire tampon
		}
	}

	function _GetCachedFile()
	{
		$this->f[$this->f_no]['create_cached_file'] = 0;

		$fp = @fopen(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['cache_name'], 'r');
		$tmp_expire = @fgets($fp, 12); // pour la position du pointeur apres le temps
		$this->f[$this->f_no]['buffer'] = @fread($fp, filesize(TPLN_CACHE_DIR.'/'.$this->f[$this->f_no]['cache_name']));
		@fclose($fp);
		clearstatcache(); // efface la mmoire tampon
	}
	
	// verifie l'exitence du repertoire de cache et le crer ou cas ou
	function _CacheDirExists()
	{ 
		// le repertoire existe ?
		if(!is_dir(TPLN_CACHE_DIR))
		{
			if(!@mkdir(TPLN_CACHE_DIR, 0755))
			{ 
				// $this->_AddTraceMsg();
				$this->_Error(7);
				return;
			}
			clearstatcache();
		}
	}
	// methode generique servant  TPLN ************************************************************************
	function extractStr($str, $start, $end, $inc_balise = 0)
	{
		$pos_start = strpos($str, $start);

		if($inc_balise)
			$pos_end = strpos($str, $end, ($pos_start));
		else
			$pos_end = strpos($str, $end, ($pos_start + strlen($start)));

		if(($pos_start !== false) && ($pos_end !== false))
		{
			if($inc_balise)
			{
				$pos1 = $pos_start;
				$pos2 = ($pos_end + strlen($end)) - $pos1;
			}
			else
			{
				$pos1 = $pos_start + strlen($start);
				$pos2 = $pos_end - $pos1;
			}

			return substr($str, $pos1, $pos2);
		}
	}

	function str_replace_count($search, $replace, $subject, $times)
	{
		$subject_original = $subject;

		$len = strlen($search);
		$pos = 0;
		for ($i = 1;$i <= $times;$i++)
		{
			$pos = strpos($subject, $search, $pos);

			if($pos === false) break;
			
			$subject = substr($subject_original, 0, $pos);
			$subject .= $replace;
			$subject .= substr($subject_original, $pos + $len);
			$subject_original = $subject;
			
		}

		return($subject);
	}

	function array2json($arr) 
	{
		$parts = array();
		$is_list = false;

		//Find out if the given array is a numerical array
		$keys = array_keys($arr);
		$max_length = count($arr)-1;
		
		if($max_length < 0) return '{}';
		
    	if(($keys[0] == 0) && ($keys[$max_length] == $max_length)) 
		{//See if the first key is 0 and last key is length - 1
        	$is_list = true;
        	for($i=0; $i<count($keys); $i++) 
			{ //See if each key correspondes to its position
            	if($i != $keys[$i]) 
				{ //A key fails at position check.
                	$is_list = false; //It is an associative array.
                	break;
            	}
        	}
    	}

		foreach($arr as $key=>$value) 
		{
        	if(is_array($value)) 
			{ //Custom handling for arrays
            	if($is_list) 
					$parts[] = $this->array2json($value); /* :RECURSION: */
            	else 
					$parts[] = '"' . $key . '":' . $this->array2json($value); /* :RECURSION: */
        	} 
			else 
			{
            	$str = '';
	            if(!$is_list) $str = '"' . $key . '":';

    	        //Custom handling for multiple data types
        	    if(is_numeric($value)) $str .= $value; //Numbers
            	elseif($value === false) $str .= 'false'; //The booleans
	            elseif($value === true) $str .= 'true';
    	        else $str .= '"' . addslashes($value) . '"'; //All other things
        	    // :TODO: Is there any more datatype we should be in the lookout for? (Object?)

				$parts[] = $str;
        	}
    	}
    	$json = implode(',',$parts);
    
    	if($is_list) return '[' . $json . ']';//Return numerical JSON
    	return '{' . $json . '}';//Return associative JSON
	}
	
	
	function getAjaxBloc($bloc_name, $out='')
	{
		if(empty($out))	$out = $this->Output();
	
		$pattern = "<!-- ajax::$bloc_name -->(.*)?<!-- /ajax::$bloc_name -->";
		@preg_match("#$pattern#msU", $out, $matches);
		
		if(count($matches) >= 2)
			return $matches[1];
		else
			return '';
	}
	
	// converts a date to MySQL
	function date2DB($date)
	{
		// date
		if(strlen($date) == 10)
		{
			list($d, $m, $Y) = explode('/', $date);
			$new_date = "$Y-$m-$d";
		}
		// datetime
		else
		{
			list($d, $m, $Y) = explode('/', $date);
			$tmp = explode(' ', $Y);
			$Y = $tmp[0];
			list($H, $i) = explode(':', $tmp[1]);
			
			$new_date = "$Y-$m-$d $H:$i:00";
		}
		
		return $new_date;
	}
	
	// converts MySQL date to form
	function DB2Date($date)
	{
		if(empty($date) || $date == '0000-00-00' || $date == '0000-00-00 00:00:00')return '';
	
		// date
		if(strlen($date) == 10)
		{
			list($Y, $m, $d) = explode('-', $date);
			$new_date = "$d/$m/$Y";
		}
		else
		{
			list($Y, $m, $d) = explode('-', $date);
			$tmp = explode(' ', $d);
			$d = $tmp[0];
			list($H, $i) = explode(':', $tmp[1]);
			
			$new_date = "$d/$m/$Y $H:$i";
		}
		
		return $new_date;
	}

	function clickable($ret)
	{
		/*$str = eregi_replace("([_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)+)",
						   "<a href=\"mailto:\\1\">\\1</a>", $str);
		$str = eregi_replace("([[:alnum:]]+)://([^[:space:]]*)([[:alnum:]#?/&=])",
							"<a href=\"\\1://\\2\\3\" target=\"_blank\">\\1://\\2\\3</a>", $str);
		*/
		
		// matches an "xxxx://yyyy" URL at the start of a line, or after a space.
		// xxxx can only be alpha characters.
		// yyyy is anything up to the first space, newline, comma, double quote or <
		$ret = preg_replace("#(^|[\n ])([\w]+?://[\w\#$%&~/.\-;:=,?@\[\]+]*)#is", "\\1<a href=\"\\2\" target=\"_blank\">\\2</a>", $ret);	
		
		// matches a "www|ftp.xxxx.yyyy[/zzzz]" kinda lazy URL thing
		// Must contain at least 2 dots. xxxx contains either alphanum, or "-"
		// zzzz is optional.. will contain everything up to the first space, newline,
		// comma, double quote or <.
		$ret = preg_replace("#(^|[\n ]|)((www|ftp)\.[\w\#$%&~/.\-;:=,?@\[\]+]*)#is", "\\1<a href=\"http://\\2\" target=\"_blank\">\\2</a>", $ret);
		
		// matches an email@domain type address at the start of a line, or after a space.
		// Note: Only the followed chars are valid; alphanums, "-", "_" and or ".".
		$ret = preg_replace("#(^|[\n ])([a-z0-9&\-_.]+?)@([\w\-]+\.([\w\-\.]+\.)*[\w]+)#i", "\\1<a href=\"mailto:\\2@\\3\">\\2@\\3</a>", $ret);
		
		// Remove our padding..
		// $ret = substr($ret, 1);
		
		
								
		return $ret;
	}		
	
} 


?>