<?php

class Mail extends DBTable
{
	var $from = '';
	var $fromLabel = '';

	var $replyTo = '';
	
	var $to = '';
	var $cc = '';
	var $bcc = '';

	var $subject = '';
	var $body = '';
	var $format = '';
	var $files = array();
	var $charset = 'iso-8859-1';
	
	var $urgent = false;
	var $confirm  = false;
	
	var $mailErr = '';


	function isMail($address)
	{
		if (preg_match('`([[:alnum:]]([-_.]?[[:alnum:]])*@[[:alnum:]]([-_.]?[[:alnum:]])*\.([a-z]{2,4}))`', $address))
            return true;
        else
			return false;
	}

	function mailUrgent($bool)
	{
		$this->urgent = $bool;
	}

	function mailCharset($str)
	{
		$this->charset = $str;
	}

	function mailConfirm($bool)
	{
		$this->confirm = $bool;
	}
	
	function mailFrom($str, $label='')
	{
		$this->from = $str;
		$this->fromLabel = $label;
	}
	
	function mailReplyTo($str)
	{
		$this->replyTo = $str;
	}
	
	
	function mailTo($str)
	{
		$this->to = $str;
	}
	
	function mailCC($str)
	{
		$this->cc = $str;
	}
	
	function mailBCC($str)
	{
		$this->bcc = $str;
	}
	

	
	function mailSubject($str)
	{
		$this->subject = $str;
	}
	
	function mailBody($str, $format)
	{
		$this->body = $str;
		$this->format = $format;
	}
	
	function mailAttachFile($src, $name, $type='')
	{
	
		// type de contenu non defini 
		if(empty($type))
		{ 
			// on essaie de reconnaitre l'extension 
			switch(strrchr(basename($name), "."))
			{ 
				case ".gz": $type = "application/x-gzip"; break; 
				case ".tgz": $type = "application/x-gzip"; break; 
				case ".zip": $type = "application/zip"; break; 
				case ".pdf": $type = "application/pdf"; break; 
				case ".png": $type = "image/png"; break; 
				case ".gif": $type = "image/gif"; break; 
				case ".jpg": $type = "image/jpeg"; break; 
				case ".txt": $type = "text/plain"; break; 
				case ".htm": $type = "text/html"; break; 
				case ".html": $type = "text/html"; break; 
				default: $type = "application/octet-stream"; break; 
			}
		} 
	
		$this->files[] = array(
								'src' => $src,
								'name' => $name,
								'type' => $type
							  );
	}
	
	function mailError()
	{
		return 'Error: '.$this->mailErr;
	}

	function mailSend()
	{
		// OBLIGATOIRE FROM
		if(!$this->isMail($this->from))
		{
			$this->mailErr = "$this->from is not a valid email address";
			return false;
		}
		
		// OBLIGATOIRE TO
		$arr = explode(',', $this->to);
		foreach($arr as $to)
		{
			$to = trim($to);
			if(!$this->isMail($to))
			{
				$this->mailErr = "$to is not a valid email address in To";
				return false;
			}
		}
		
		// VERIF REPLYTO
		if(!empty($this->replyTo))
		{
			if(!$this->isMail($this->replyTo))
			{
				$this->mailErr = "$this->replyTo is not a valid email address";
				return false;
			}
		}

		
	
		// headers
		$headers = '';
		
		if(!empty($this->fromLabel))$this->from = "$this->fromLabel <$this->from>";
		$headers .= "From: $this->from"."\n";
		if(count($this->files) > 0)$headers .= "MIME-Version: 1.0\n";
			
		//$headers .= "To: $this->to"."\n";
		$headers .= "Return-Path: $this->from"."\n";
		if(!empty($this->replyTo))$headers .= "Reply-To: $this->replyTo"."\n";
		if(!empty($this->cc))$headers .= "Cc: $this->cc"."\n";
		if(!empty($this->bcc))$headers .= "Bcc: $this->bcc"."\n";
		if(!empty($this->confirm))$headers .= "Disposition-Notification-To: $this->from"."\n";
		$headers .= 'X-Mailer: PHP/'.phpversion()."\n";
		
		if($this->urgent)$headers .= "X-Priority: 1\n";
		
		// y a des fichiers ?
		if(count($this->files) > 0)
		{	
			// on ajoute le message
			if($this->format == 'HTML')
				$this->mailAttachFile($this->body, '', 'text/html');
			else
				$this->mailAttachFile($this->body, '', 'text/plain');
			
			$boundary = "b".md5(uniqid(time())); 
			$headers .= "Content-Type: multipart/mixed; boundary = $boundary\n\nThis is a MIME encoded message.\n\n--$boundary"; 

			// on construit le MIME  l'envers
			for($i=count($this->files)-1; $i >=  0; $i--)
			{
				// on construit le message
				$headers .= "\n"; 			
					$message = $this->files[$i]['src']; 
					$message = chunk_split(base64_encode($message)); 
				$headers .= "Content-Type: ".$this->files[$i]['type'];
				if(!empty($this->files[$i]['name']))
					$headers .= "; name = \"".$this->files[$i]['name']. "\"";
				$headers .= "\nContent-Transfer-Encoding: base64\n\n";
				if(!empty($this->files[$i]['name']))
					$headers .= "Content-Disposition: attachment;\n\n";
				$headers .= "$message\n";
				$headers .= "--$boundary";
			}
			$headers .= "--";
			
			
			// envoie
			if(!@mail($this->to, $this->subject, '', $headers))
			{
				$this->mailErr = "Email has not been sent";
				return false;
			}	
		}
		else
		{
			// on envoie en HTML ?
			$headers .= "MIME-Version: 1.0\n";  
			if($this->format == 'HTML')
				$headers .= "content-type: text/html; charset=$this->charset\n";  
			
			if(!@mail($this->to, $this->subject, $this->body, $headers))
			{
				$this->mailErr = "Email has not been sent";
				return false;
			}
		}
		
		return true;
			
	}
}

?>